@@ -226,7 +226,7 @@ def check_zmat(atoms=None,rmin=0.8,rmax=1.35,angmax=25.0,
 
 
 class AtomDance(object):
-  def __init__(self,atoms=None,poscar=None,nn=True,ffield='ffield.json',
+  def __init__(self,atoms=None,poscar=None,nn=True,nomb=False,ffield='ffield.json',
                rotAng=40.0,angmax=30.0,freeatoms=None,FirstAtom=None,
                rmin=0.4,rmax=1.25,botol=0.0):
       self.rmin          = rmin
@@ -245,6 +245,7 @@ class AtomDance(object):
       self.ir = IRFF_NP(atoms=atoms,
                         libfile=ffield,
                         rcut=None,
+                        nomb=nomb,
                         nn=nn)
       self.natom     = self.ir.natom
       self.atom_name = self.ir.atom_name
@@ -308,9 +309,10 @@ class AtomDance(object):
       self.zmat_index = []
       self.zmat_id    = []
       specs           = atoms.get_chemical_symbols()
-      self.mols       = molecules(self.natom,specs,atoms.positions,
+      self.mols       = molecules(self.natom,specs,X=atoms.positions,
                                   cell=atoms.cell,
-                                  table=self.neighbors)
+                                  table=self.neighbors,
+                                  sizeiscell=False)
       for m in self.mols:
           if len(m.mol_index)==1:
              self.zmat_index.append([-1,-1,-1])
@@ -396,6 +398,8 @@ class AtomDance(object):
          ry  = np.sqrt(np.sum(np.square(vy)))
          if ry>0.00000001:  # paralell
             uy = vy/ry
+         else:
+            uy = np.array([0.0,0.0,1.0])
          #print(uy)
       else:
          vkl = atoms.positions[k] - atoms.positions[l] 
@@ -405,9 +409,9 @@ class AtomDance(object):
          vy  = ukl - rk*ux
          ry  = np.sqrt(np.sum(np.square(vy)))
          if ry>0.00000001:
-            uy = np.array([0.0,0.0,1.0])
-         else:
             uy  = vy/ry 
+         else:
+            uy = np.array([0.0,0.0,1.0])
 
       if ry>0.00000001 and abs(ang - 180.0)>0.00000001:
          a   = ang*3.141593/180.0
@@ -1316,8 +1320,8 @@ class AtomDance(object):
              print('Atoms too closed.')
              break
              
-          calc = SinglePointCalculator(atoms_,energy=self.ir.E)
-          atoms_.set_calculator(calc)
+          atoms_.calc = SinglePointCalculator(atoms_,energy=self.ir.E)
+          # atoms_.set_calculator(calc)
           images.append(atoms_)
           if not traj is None: his.write(atoms=atoms_)
       return images
