@@ -8,8 +8,8 @@ from ase.constraints import FixAtoms
 import matplotlib.pyplot as plt
 from math import ceil
 from .molecule import packmol
-from .dft.siesta import siesta_md # siesta_opt 
-from .dft.qe import qemd # qeopt  
+from .dft.siesta import siesta_md,siesta_opt 
+from .dft.qe import qemd,qeopt  
 from .dft.SinglePointEnergy import SinglePointEnergies
 from .dft.CheckEmol import check_emol
 #from .md.gulp import nvt as gulp_nvt
@@ -51,13 +51,13 @@ def get_zpe(p,atom_name):
 
 class LearningMachine(object):
   ''' recursive learning machine constructed by I-ReaxFF and siesta '''
-  def __init__(self,initConfig='poscar.gen',
+  def __init__(self,config='poscar.gen',
                dataset=None,ncpu=40,
                maxiter=100,
                dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
                batch=50,max_batch=100,
                step=20000,md_step=20,MaxMDstep=200,MinMDstep=50,
-               colFrame=50,colmin=3,
+               col_frame=50,col_min_interval=3,
                mom_step=100,
                mpopt=[1,1,1,1], messages=1,
                optword='nocoul',cons=None,clip={},
@@ -66,20 +66,21 @@ class LearningMachine(object):
                convergence=0.01,nconvergence=3,
                accCriteria=0.9,lossCriteria=10.0,lossTole=1.0,accTole=0.05,
                accMax=0.9,accInc=1.0001,mdInc=1.2,resetAcc=2,
-               learnWay=5,FreePairs=None,beta=None,
-               FreeAtoms=None,FirstAtom=None,
+               learn_method=3,lattice_opt=False,FreePairs=None,beta=None,
+               free_atoms=None,first_atom=None,
                rodic=None,
                rmin=0.88,rmax=1.33,angmax=25.0,
-               CheckZmat=True,
+               CheckZmat=True,uncertainty=0.96,period=30,
                EngTole=0.05,dEtole=0.2,dEstop=2.0,
                nn=True,vdwnn=False,
                bo_layer=None,mf_layer=[9,1],be_layer=[9,1],vdw_layer=None,#[6,1],
-               be_universal_nn=None,bo_universal_nn=None,mf_universal_nn=None,vdw_universal_nn=None,
+               be_universal_nn=None,bo_universal_nn=None,
+               mf_universal_nn=None,vdw_universal_nn=None,
                BOFunction=0,EnergyFunction=1,MessageFunction=3,VdwFunction=1,
                spv_be=False,beup={},belo={},
-               spv_bo=False,
+               bo_clip=False,
                spv_vdw=False,vup={},vlo={},
-               spv_ang=False,lambda_ang=0.02,
+               pi_clip=False,# lambda_ang=0.02,
                weight={'others':2.0},
                lambda_bd=10000.0,lambda_reg=0.0001, # regularize=True,
                lambda_me=0.1,learning_rate=0.0001,
@@ -90,8 +91,8 @@ class LearningMachine(object):
               batch: batch size
       '''
       self.kwargs         = kwargs
-      self.initConfig     = initConfig
-      self.label          = initConfig.split('.')[0]
+      self.config         = config
+      self.label          = config.split('.')[0]
       self.aidir          = 'aimd_' + self.label
       self.accCriteria    = accCriteria # accuracy convergence criteria
       self.lossCriteria   = lossCriteria    
@@ -104,10 +105,13 @@ class LearningMachine(object):
       self.rmax           = rmax
       self.angmax         = angmax
       self.CheckZmat      = CheckZmat
+      self.uncertainty    = uncertainty
+      self.period         = period
       self.convergence    = convergence
       self.cons           = cons
       self.nconvergence   = nconvergence
-      self.learnWay       = learnWay    # the way of learing
+      self.learn_method   = learn_method     # the way of learing
+      self.lattice_opt    = lattice_opt
       self.ncpu           = ncpu
       self.step           = step
       self.batch          = batch
@@ -127,8 +131,8 @@ class LearningMachine(object):
       self.md_step        = md_step
       self.mom_step       = mom_step
       self.beta           = beta
-      self.colFrame       = colFrame
-      self.colmin         = colmin
+      self.col_frame       = col_frame
+      self.col_min_interval = col_min_interval
       self.EngTole        = EngTole
       self.dEtole         = dEtole
       self.dEstop         = dEstop
@@ -155,31 +159,28 @@ class LearningMachine(object):
       self.vdw_universal_nn= vdw_universal_nn
       self.beup           = beup
       self.belo           = belo
-      #self.boc            = boc
-      #self.bolo          = bolo
       self.vup            = vup
       self.vlo            = vlo
-      self.spv_bo         = spv_bo
+      self.bo_clip        = bo_clip
       self.spv_vdw        = spv_vdw
       self.spv_be         = spv_be
-      self.spv_ang        = spv_ang
+      self.pi_clip         = pi_clip
       self.lambda_me      = lambda_me
       self.weight         = weight
       self.lambda_bd      = lambda_bd
       # self.regularize   = regularize
       self.lambda_reg     = lambda_reg
-      self.lambda_ang     = lambda_ang
       self.learning_rate  = learning_rate
       self.BOFunction     = BOFunction
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
       self.writelib       = writelib
-      self.freeatoms      = FreeAtoms
+      self.freeatoms      = free_atoms
       self.freepairs      = FreePairs
-      self.a              = AtomDance(poscar=self.initConfig,nn=self.nn,
+      self.a              = AtomDance(poscar=self.config,nn=self.nn,
                                     ffield=ffield,
                                     rmin=self.rmin-0.05,rmax=self.rmax,
-                                    FirstAtom=FirstAtom,freeatoms=self.freeatoms)
+                                    FirstAtom=first_atom,freeatoms=self.freeatoms)
       if self.freepairs is None:
          self.a.get_freebond(freeatoms=self.freeatoms)
       else:
@@ -239,7 +240,7 @@ class LearningMachine(object):
       data_dir     = {}
       running      = True
       Deformed     = 0
-      if self.learnWay==5:
+      if self.learn_method==5:
          if self.freeatoms is None:
             self.freeatoms = []
          ms        = len(self.freeatoms)
@@ -257,13 +258,13 @@ class LearningMachine(object):
       images         = None
       zvlo,zvhi      = 0.0,0.0
       u_zvlo,u_zvhi  = 0.0,0.0
-      relax_step     = 5*self.colmin
+      relax_step     = 5*self.col_min_interval
 
       while running:
           optlog   = None
           relaxlog = None
-          learnWay = self.learnWay
-          mom_step = self.mom_step*(ms+1) if learnWay==5 else self.mom_step
+          learn_method = self.learn_method
+          mom_step = self.mom_step*(ms+1) if learn_method==5 else self.mom_step
           run_dir  = self.aidir+'/'+self.label+'-'+str(iter_)
           data_dir[self.label+'-'+str(iter_)] = cwd+'/'+run_dir+'/'+self.label+'.traj'
 
@@ -271,28 +272,29 @@ class LearningMachine(object):
              iter_ += 1
              continue
           if images is not None:
-             atoms = images[-1]   
+             atoms = images[-1] 
           elif isfile('md.traj'): 
-             atoms = read('md.traj',index=-1)
+             atoms = read('md.traj',index=-1) 
              consistance = self.check_atoms(atoms)
-             assert consistance,'-  The species or cell parameter in md.traj is not consistant with initial configuration!'
+             assert consistance,'-  The species in md.traj is not consistant with initial configuration!'
              # print('-  atomic structure from MD trajectories.')
           else:
-             print('-  cannot find MD trajectory, use learnWay=1 in the first iter.')
-             atoms = read(self.initConfig)
-             learnWay = 1
+             print('-  cannot find MD trajectory, use learn_method=1 in the first iter.')
+             atoms = read(self.config)
+             learn_method = 4 if self.learn_method==4 else 1
           if not self.freeatoms is None:
              atoms = self.a.check_momenta(atoms,freeatoms=self.freeatoms)
              
-          if learnWay in [4,5,6]:
+          if learn_method in [4,5,6]:
              momsteps += mdsteps
              if momsteps >= mom_step:
                 momsteps  = 0
-                if learnWay == 4:
-                   atoms,self.learnpair,groupi,groupj = self.a.bond_momenta(atoms)
-                   if self.learnpair is None:
-                      learnWay = 3
-                elif learnWay == 5:
+                if learn_method == 4:
+                   # atoms,self.learnpair,groupi,groupj = self.a.bond_momenta(atoms)
+                   # if self.learnpair is None:
+                   #    learn_method = 3
+                   self.CheckZmat=False
+                elif learn_method == 5:
                    if ms<self.natom:
                       i = self.a.zmat_id[ms]
                       if i not in self.freeatoms: 
@@ -313,24 +315,25 @@ class LearningMachine(object):
                          # self.learnpair = [i,j]
                          atoms,_,_ = self.a.set_bond_momenta(i,j,atoms)
                       else:
-                         learnWay = 3
+                         learn_method = 3
                          self.learnpair = None
-                   if learnWay ==5: learnWay = 4
-                # learnWay  = 2
+                   if learn_method ==5: learn_method = 3 # 4
+                # learn_method  = 2
                 ms       += 1
 
           # self.get_fixatom()
           # atoms.set_constraint(self.fixatoms)
           # atoms.write('poscar.gen')                                    # for aimd
 
-          if learnWay==1: 
+          if learn_method<=2: 
              dft_step = self.dft_step
           elif Deformed>=1.0 or uncertainty_zv is not None:
-             dft_step = relax_step
+             dft_step = self.dft_step
           else:
-             dft_step = int(mdsteps/self.colFrame)+1
-          
-          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,learnWay)
+             dft_step = self.col_frame # int(mdsteps/self.col_frame)+1
+
+          ### aimd: DO ab init calculations
+          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,learn_method) 
           e_siesta.append(e_aimd[0])                                   # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
@@ -373,9 +376,8 @@ class LearningMachine(object):
                                                      MessageFunction=self.MessageFunction,
                                                      # bore=self.bore, # bom=self.bom,
                                                      spv_be=self.spv_be,belo=self.belo,beup=self.beup,
-                                                     spv_bo=self.spv_bo,#boc=self.boc,#boup=self.boup,
+                                                     bo_clip=self.bo_clip,pi_clip=self.pi_clip,
                                                      spv_vdw=self.spv_vdw,vlo=self.vlo,vup=self.vup,
-                                                     spv_ang=self.spv_ang,lambda_ang=self.lambda_ang,
                                                      lambda_me=self.lambda_me,weight=self.weight,
                                                      lambda_reg=self.lambda_reg,
                                                      lambda_bd=self.lambda_bd,
@@ -398,21 +400,25 @@ class LearningMachine(object):
 
           if self.CheckZmat:
              # if not zmatopt is None:
-             #    optlog,_ = self.a.get_optimal_zv(atoms,zmatopt,optgen=self.initConfig)
+             #    optlog,_ = self.a.get_optimal_zv(atoms,zmatopt,optgen=self.config)
              if mdsteps>10:
-                uncertainty_zv,u_zvlo,u_zvhi,optlog = self.a.get_zmat_uncertainty(atoms,optgen=self.initConfig)
+                uncertainty_zv,u_zvlo,u_zvhi,optlog = self.a.get_zmat_uncertainty(atoms,optgen=self.config)
 
           if mdsteps<10: mdsteps = 10
           # zmats    = None
-          images     = None
+          images     = None if learn_method!=4 else [atoms]
           # zmatopt  = zmat_vairable # if Deformed>=1.0 else uncertainty_zv
 
-          if learnWay==6:
+          if learn_method==6:
              if ms<len(self.freeatoms):
                 i = self.freeatoms[ms]
                 self.a.pes(i,atoms,nbin=dft_step,dr=0.1,traj='md.traj')
              if ms == len(self.freeatoms) -1:
-                self.learnWay=3
+                self.learn_method=3
+          elif learn_method==4:
+             e_gmd,mdsteps,Deformed,zmat_variable,zvlo,zvhi = self.mlmd(atoms, 10000,
+                                      iter_,learn_method,beta=self.beta,learnpair=self.learnpair,
+                                      groupi=groupi,groupj=groupj) 
           else:
              if Deformed>=1.0:
                 images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=relax_step,
@@ -422,7 +428,7 @@ class LearningMachine(object):
                                                     zmatrix=self.a.zmatrix,zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj') # ,reset=True
              else:
                 e_gmd,mdsteps,Deformed,zmat_variable,zvlo,zvhi = self.mlmd(atoms, 10000,
-                                                iter_,learnWay,beta=self.beta,learnpair=self.learnpair,
+                                                iter_,learn_method,beta=self.beta,learnpair=self.learnpair,
                                                 groupi=groupi,groupj=groupj) 
                 if self.CheckZmat and mdsteps<=3:
                    mdsteps= 10
@@ -447,14 +453,14 @@ class LearningMachine(object):
 
           self.save_config()
 
-          e_gmd = eml_ if learnWay in [3,4,5] else 0.0
+          e_gmd = eml_ if learn_method in [3,4,5] else 0.0
           e_gulp.append(e_gmd[0])
           diff  = abs(e_gmd[0]-e_aimd[0])
           it.append(iter_)
 
           plot_energies(it,e_siesta,e_gulp)                             # plot learning status                                   
           self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,LabelDataLog,
-          	               learnWay,Deformed,
+          	               learn_method,Deformed,
                            self.a.InitZmat,uncertainty_zv,zvlo,zvhi,optlog,relaxlog)
           converg_ = converg
           if diff<=self.convergence: 
@@ -481,7 +487,7 @@ class LearningMachine(object):
           iter_ += 1
 
   def learninglog(self,i,loss,accu,mdsteps,eirff,edft,dEmax,d2Emax,LabelDataLog,
-                  learnWay,Deformed,
+                  learn_method,Deformed,
                   InitZmat,uz,uzlo,uzhi,optlog,relaxlog):
       with open('learning.log','a') as l:
          l.write('                  ------------------------------------                  \n')
@@ -509,7 +515,7 @@ class LearningMachine(object):
          l.write('MD Steps:  {:d}  \n'.format(mdsteps))
          l.write('Energy from ML and DFT:  {:f}  .vs.  {:f} \n'.format(eirff,edft))
          l.write('The max first derivative and second derivative:  {:f}  {:f} \n'.format(dEmax,d2Emax))
-         l.write('The learning method of this iter:  {:d} \n'.format(learnWay))
+         l.write('The learning method of this iter:  {:d} \n'.format(learn_method))
          if not relaxlog is None:
             if Deformed>=1:
                l.write('Molecule structure is deformed, using zmatmix relax\n' )
@@ -520,7 +526,7 @@ class LearningMachine(object):
             l.write(optlog)
          l.write(' \n')
 
-  def aimd(self,atoms,cwd,run_dir,tstep,learnWay):
+  def aimd(self,atoms,cwd,run_dir,tstep,learn_method):
       mkdir(run_dir)
       chdir(cwd+'/'+run_dir)
       
@@ -536,31 +542,53 @@ class LearningMachine(object):
       dEmax  = 0.0
       d2Emax = 0.0
       
-      if learnWay==1:
+      if learn_method==1:
          if self.dft=='siesta':
             images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
                                atoms=atoms,FreeAtoms=self.freeatoms,
                                xcf=self.xcf,xca=self.xca,basistype=self.basistype,**self.kwargs)
-            # images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=False,us='F',
-            #                     tstep=tstep,gen=cwd+'/'+gen,
-            #                     xcf=self.xcf,xca=self.xca,basistype=self.basistype)
             ind_   = 'Molecular Dynamics Simulation'
          elif self.dft=='qe':
             images = qemd(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,tstep=tstep,
                           atoms=atoms,kpts=self.kpts,**self.kwargs)      ## geomentry optimize
-            # images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
-            #                gen=cwd+'/'+gen,
-            #                kpts=self.kpts,**self.kwargs) ## geomentry optimize
             ind_   = 'Geomentry Optimization'
          else:
             raise RuntimeError('-  This method not implimented!')
          eaimd = [images[0].get_potential_energy()]
          system('cp %s ../../md.traj' %(self.label+'.traj'))
-      elif learnWay>=2:
+      elif learn_method==2:
+         if self.dft=='siesta':
+            images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=self.lattice_opt,us='F',
+                                tstep=tstep,FreeAtoms=self.freeatoms,
+                                xcf=self.xcf,xca=self.xca,basistype=self.basistype,**self.kwargs)
+            system('cp {:s}.traj md.traj'.format(self.label))
+            E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies(traj='md.traj',label=self.label,EngTole=self.EngTole,
+                                                 frame=tstep,select=True,
+                                                 dE=self.dEtole,colmin=self.col_min_interval,
+                                                 dft=self.dft,kpts=self.kpts,
+                                                 xcf=self.xcf,xca=self.xca,basistype=self.basistype,
+                                                 cpu=self.ncpu,**self.kwargs)
+            ind_   = 'Geomentry Optimization'
+         elif self.dft=='qe':
+            images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,tstep=tstep,
+                           kpts=self.kpts,**self.kwargs) ## geomentry optimize
+            ind_   = 'Geomentry Optimization'
+            system('cp {:s}.traj md.traj'.format(self.label))
+            E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies(traj='md.traj',label=self.label,EngTole=self.EngTole,
+                                                 frame=tstep,select=True,
+                                                 dE=self.dEtole,colmin=self.col_min_interval,
+                                                 dft=self.dft,kpts=self.kpts,
+                                                 xcf=self.xcf,xca=self.xca,basistype=self.basistype,
+                                                 cpu=self.ncpu,**self.kwargs)
+         else:
+            raise RuntimeError('-  This method not implimented!')
+         eaimd = [images[0].get_potential_energy()]
+         system('cp %s ../../md.traj' %(self.label+'.traj'))
+      elif learn_method>2:
          # print('-  files in this dir \n',cwd+'/'+run_dir,'\n',listdir())
          E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies(traj='md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
-                                                 dE=self.dEtole,colmin=self.colmin,
+                                                 dE=self.dEtole,colmin=self.col_min_interval,
                                                  dft=self.dft,kpts=self.kpts,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
                                                  cpu=self.ncpu,**self.kwargs)
@@ -571,19 +599,23 @@ class LearningMachine(object):
       chdir(cwd)
       return eaimd,emlmd,dEmax,d2Emax,ind_
 
-  def mlmd(self,atoms,Tmax,Iter,learnWay,learnpair=None,beta=None,
+  def mlmd(self,atoms,Tmax,Iter,learn_method,learnpair=None,beta=None,
            groupi=None,groupj=None):        
       ''' run classic MD to test training results '''
-      mdstep = max(int(self.md_step/5),2) if learnWay==2 else self.md_step
+      mdstep = max(int(self.md_step/5),2) if learn_method==2 else self.md_step
       zmats  = None
+      nomb = False if self.optword.find('nomb')<0 and learn_method!=4 else True
+      active = True if learn_method==4 else False
       irmd = IRMD(atoms=atoms,label=self.label,Iter=Iter,initT=self.T,
                   time_step=self.dt_mlmd,totstep=mdstep,Tmax=Tmax,
                   ro=self.ro,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
                   CheckZmat=self.CheckZmat,InitZmat=self.a.InitZmat,
                   zmat_id=self.a.zmat_id,zmat_index=self.a.zmat_index,
                   dEstop=self.dEstop,dEtole=self.dEtole,nn=self.nn,vdwnn=self.vdwnn,
-                  learnpair=learnpair,beta=beta,groupi=groupi,groupj=groupj,freeatoms=self.freeatoms)
-      if learnWay==2:
+                  learnpair=learnpair,beta=beta,groupi=groupi,groupj=groupj,
+                  nomb=nomb,active=active,uncertainty=self.uncertainty,period=self.period,
+                  freeatoms=self.freeatoms)
+      if learn_method==2:
          Deformed,zmats,zv,zvlo,zvhi = irmd.opt()
       else:
          Deformed,zmats,zv,zvlo,zvhi = irmd.run()
@@ -654,13 +686,6 @@ class LearningMachine(object):
       with open('options.json','w') as fj:
            options = {'accCriteria':self.accCriteria,
                     'lossCriteria':self.lossCriteria,
-                    # 'lossTole':self.lossTole,
-                    # 'rmax':self.rmax,
-                    # 'angmax':self.angmax,
-                    # 'accTole':self.accTole,
-                    # 'accMax':self.accMax,
-                    # 'accInc':self.accInc,
-                    # 'resetAcc':self.resetAcc,
                     'step':self.step,
                     'md_step':self.md_step,
                     # 'mom_step':self.mom_step,
@@ -670,27 +695,18 @@ class LearningMachine(object):
                     'dEtole':self.dEtole,
                     'dEstop':self.dEstop,
                     'beta':self.beta,
-                    # 'mpopt':self.mpopt,
-                    # 'regularize':self.regularize,
                     'lambda_reg':self.lambda_reg,
                     'lambda_bd':self.lambda_bd,
                     'lambda_me':self.lambda_me,
-                    'lambda_ang':self.lambda_ang,
                     'learning_rate':self.learning_rate,
-                    # 'rmin':self.rmin,
                     'T':self.T,
                     'maxiter':self.maxiter,
-                    # 'max_batch':self.max_batch,
-                    #'optword':self.optword,
-                    'learnWay':self.learnWay,
-                    'colFrame':self.colFrame,
+                    'learn_method':self.learn_method,
+                    'col_frame':self.col_frame,
                     'CheckZmat':self.CheckZmat,
                     'freeatoms':self.freeatoms}
-                    # 'freepairs':self.freepairs
-                    # 'bore':self.bore,
-                    # 'writelib':self.writelib
 
-           if self.learnWay==1:
+           if self.learn_method==1:
               options['dft_step'] = self.dft_step
            js.dump(options,fj,sort_keys=True,indent=2) 
 
@@ -699,20 +715,13 @@ class LearningMachine(object):
            options = js.load(fj)
            self.accCriteria  = options['accCriteria']
            self.lossCriteria = options['lossCriteria']
-           # self.lossTole     = options['lossTole']
-           # self.rmax         = options['rmax']
-           # self.angmax       = options['angmax']
-           # self.accTole      = options['accTole']
-           # self.accMax     = options['accMax']
-           # self.accInc     = options['accInc']
-           # self.resetAcc   = options['resetAcc']
            self.step         = options['step'] 
-           if self.learnWay==1:
+           if self.learn_method==1:
               self.dft_step     = options['dft_step'] 
            self.md_step      = options['md_step'] 
            
            if 'mom_step' in options: self.mom_step  = options['mom_step'] 
-           if 'colmin'   in options: self.colmin = options['colmin'] 
+           if 'col_min_interval'   in options: self.col_min_interval = options['col_min_interval'] 
 
            self.MaxMDstep    = options['MaxMDstep'] 
            self.MinMDstep    = options['MinMDstep'] 
@@ -720,24 +729,15 @@ class LearningMachine(object):
            self.dEtole       = options['dEtole'] 
            self.dEstop       = options['dEstop'] 
            self.beta         = options['beta'] 
-           # self.mpopt        = options['mpopt'] 
-           # self.regularize   = options['regularize'] 
            self.lambda_reg   = options['lambda_reg'] 
            self.lambda_bd    = options['lambda_bd']
            self.lambda_me    = options['lambda_me']
-           self.lambda_ang   = options['lambda_ang']
-           # self.rmin       = options['rmin'] 
            self.T            = options['T'] 
            self.maxiter      = options['maxiter']
-           # self.max_batch  = options['max_batch']
-           #self.optword      = options['optword']
-           self.learnWay     = options['learnWay']
+           self.learn_method     = options['learn_method']
            self.learning_rate= options['learning_rate']
-           self.colFrame     = options['colFrame']
+           self.col_frame     = options['col_frame']
            self.CheckZmat    = options['CheckZmat']
-           # self.bore       = options['bore']
-           # self.writelib   = options['writelib']
-           # self.freepairs    = options['freepairs']
            if self.freeatoms!= options['freeatoms']:
               self.freeatoms = options['freeatoms']
               self.a.get_freebond(self.freeatoms)
@@ -752,18 +752,18 @@ if __name__ == '__main__':
    dataset = {'nm6_5':'/home/feng/siesta/nm6_5',
               'nm6_14':'/home/feng/siesta/nm6_14' }
 
-   lm = LearningMachine(initConfig='c14.gen',
+   lm = LearningMachine(config='c14.gen',
                      dataset=dataset,ncpu=8,
                      maxiter=1,
-                     step=50000,md_step=100,MaxMDstep=1000,mom_step=50,colFrame=100,
+                     step=50000,md_step=100,MaxMDstep=1000,mom_step=50,col_frame=100,
                      T=500,
                      angmax=20.0,
                      lossCriteria=20.0,lossTole=1.0,
                      CheckZmat=False,
                      # regularize=True,
                      lambda_reg=0.001,lambda_bd=1000.0,lambda_me=0.001,lambda_ang=0.01,
-                     learnWay=1,dft_step=50,
-                     beta=0.77,# FirstAtom=4,FreeAtoms=[4,5,8], 
+                     learn_method=1,dft_step=50,
+                     beta=0.77,# first_atom=4,FreeAtoms=[4,5,8], 
                      EngTole=0.01,dEtole=0.05,dEstop=2.0,
                      EnergyFunction=1,
                      bo_layer=[4,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=[6,1],
@@ -771,7 +771,6 @@ if __name__ == '__main__':
                      be_universal_nn=None,#['O-O'],
                      mf_universal_nn=None,#['C'],
                      spv_be=False,bore={'others':0.22},
-                     spv_ang=False,
                      vdw_universal_nn=None,vdwnn=True,
                      weight={# 'no2-l':20.0,
                              'others':2.0},
