@@ -20,6 +20,7 @@ def setRcut(bonds,rcut_,rcuta_,re_):
               'O-O':2.5,'O-H':2.0,'O-F':2.5,'O-Al':3.2,
               'H-H':2.0,'F-H':2.0,'H-Al':2.5,
               'F-F':2.5,'F-Al':3.2,
+              'Fe-Fe':3.8,'Fe-O':2.8,
               'Al-Al':3.8,
               'others':2.5}
     else:
@@ -28,14 +29,15 @@ def setRcut(bonds,rcut_,rcuta_,re_):
     # in princeple, the cutoff should 
     # cut the first nearest neithbors
     if rcuta_ is None:
-       rcuta = rcut
-       # rcuta = {'C-C':1.95,'C-H':1.75,'C-N':1.95,'C-O':1.95,'C-F':1.95,'C-Al':2.9,
-       #         'N-N':1.95,'N-O':1.95,'N-H':1.75,'N-F':1.95,'N-Al':2.8,
-       #         'O-O':1.95,'O-H':1.75,'O-F':1.95,'O-Al':2.8,
-       #         'H-H':1.35,'F-H':1.75,'H-Al':2.5,
-       #         'F-F':1.95,'F-Al':2.8,
-       #         'Al-Al':3.4,
-       #         'others':1.95}
+       # rcuta = rcut
+       rcuta = {'C-C':1.95,'C-H':1.75,'C-N':1.95,'C-O':1.95,'C-F':1.95,'C-Al':2.9,
+               'N-N':1.95,'N-O':1.95,'N-H':1.75,'N-F':1.95,'N-Al':2.8,
+               'O-O':1.95,'O-H':1.75,'O-F':1.95,'O-Al':2.8,
+               'H-H':1.35,'F-H':1.75,'H-Al':2.5,
+               'F-F':1.95,'F-Al':2.8,
+               'Fe-Fe':3.5,'Fe-O':2.6,
+               'Al-Al':3.4,
+               'others':1.95}
     else:
        rcuta = rcuta_
 
@@ -47,6 +49,7 @@ def setRcut(bonds,rcut_,rcuta_,re_):
              'O-O':1.25,'O-H':1.07,'O-F':1.24,'O-Al':1.91,
              'H-H':0.7,'H-F':0.88,'H-Al':1.55,
              'F-F':1.16,'F-Al':1.83,
+             'Fe-Fe':2.19,'Fe-O':1.24,
              'Al-Al':2.8,
              'others':1.5}
     else:
@@ -55,7 +58,7 @@ def setRcut(bonds,rcut_,rcuta_,re_):
     # with open('ffield.json','w') as fj:
     #      js.dump(RC,fj,sort_keys=True,indent=2)
 
-    if not bonds is None:
+    if bonds is not None:
        for bd in bonds:
            b = bd.split('-')
            bdr = b[1]+'-'+b[0]
