@@ -93,7 +93,7 @@ def deb_vdw(images,i=0,j=1,show=False):
     vmin_ =  np.min(Ev)
     ev    =  np.array(EV) - vmin_# )/emin_
 
-    plt.figure(figsize=figsize)     
+    plt.figure()     
     # plt.plot(bopsi,alpha=0.8,linewidth=2,linestyle=':',color='k',label=r'$BO_p^{\sigma}$')
     # plt.plot(boppi,alpha=0.8,linewidth=2,linestyle='-.',color='k',label=r'$BO_p^{\pi}$')
     # plt.plot(boppp,alpha=0.8,linewidth=2,linestyle='--',color='k',label=r'$BO_p^{\pi\pi}$')
@@ -356,8 +356,7 @@ def deb_eang(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False,frame=[0])
     a     = 0
     
     eang,ecoa,epen = [],[],[]
-    
-    ir = IRFF_NP(atoms=images[frame],
+    ir = IRFF_NP(atoms=images[frame[0]],
                  libfile='ffield.json',
                  nn=True)
     for f_ in frame:
@@ -371,6 +370,8 @@ def deb_eang(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False,frame=[0])
     if print_: print('\n Angle specified: \n')
 
     for i_,atoms in enumerate(images):  
+        ir.calculate(atoms)
+        
         found = False 
         for na,angle in enumerate(ir.angs):  
             i_,j_,k_ = angle
@@ -380,11 +381,10 @@ def deb_eang(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False,frame=[0])
         if not found:
            # print('Warning: no angle found for {:s} in this trajector frame!'.format(ang))   
            continue 
-        ir.calculate(atoms)
 
-        eang.append(ir.Eang)    
-        ecoa.append(ir.Etcon)
-        epen.append(ir.Epen)
+        eang.append(ir.eang[a])    
+        # ecoa.append(ir.Etcon)
+        # epen.append(ir.Epen)
         if print_:
            print('{:3d}  {:6.4f}  {:6.4f} Dpi: {:6.4f} pbo: {:6.4f} N: {:6.4f} SBO3: {:6.4f} eang: {:6.4f}'.format(i_,
                      ir.thet0[a],ir.theta[a],ir.sbo[a],ir.pbo[a],
@@ -635,6 +635,120 @@ def deb_thet(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False,dFrame=[])
     if show: plt.show()
     plt.close()
 
+def get_changed_fourboday(images=None,traj='md.traj'):
+    if images is None:
+       images = Trajectory(traj)
+    Eb,Ea,Etor,E = [],[],[],[]
+    ir = IRFF_NP(atoms=images[0],
+                 libfile='ffield.json',
+                 nn='True')
+    ir.calculate(images[0])
+
+    tor_min = {}
+    tor_max = {}
+    for i,tor in enumerate(ir.tors):
+        t = tuple(tor)
+        tor_min[t] = ir.etor[i]
+        tor_max[t] = ir.etor[i]
+    # print(tor_min)
+
+    for i_,atoms in enumerate(images):       
+        ir.calculate(images[i_])
+        # print('%d Energies: ' %i_,'%12.4f ' %ir.E )
+
+        Eb.append(ir.Ebond)
+        Ea.append(ir.Eang)
+        Etor.append(ir.Etor)
+        E.append(ir.E)
+
+        for i,tor in enumerate(ir.tors):
+            t  = tuple(tor)
+            t_ = (tor[3],tor[2],tor[1],tor[0])
+            if t in tor_min:
+               if tor_min[t]>ir.etor[i]:
+                  tor_min[t] = ir.etor[i]
+               if tor_max[t]<ir.etor[i]:
+                  tor_max[t] = ir.etor[i]
+            elif t_ in tor_min:
+               if tor_min[t_]>ir.etor[i]:
+                  tor_min[t_] = ir.etor[i]
+               if tor_max[t_]<ir.etor[i]:
+                  tor_max[t_] = ir.etor[i]
+            else:
+              tor_min[t] = ir.etor[i]
+              tor_max[t] = ir.etor[i]
+    tor_change = []
+    tor_name   = []
+    m_ = 0.0
+    mi = 0
+    for i,t in enumerate(tor_min):
+        m = tor_max[t] - tor_min[t]
+        if m>m_:
+           mi = i
+           m_ = m
+        tor_change.append(m)
+        tor_name.append(t)
+    tor  = tor_name[mi]
+    tor_ = '{:s}-{:s}-{:s}-{:s}'.format(ir.atom_name[tor[0]],ir.atom_name[tor[1]],
+                                        ir.atom_name[tor[2]],ir.atom_name[tor[3]])
+    print(tor_,tor,tor_change[mi])
+
+def get_changed_threeboday(images=None,traj='md.traj'):
+    if images is None:
+       images = Trajectory(traj)
+    Eb,Ea,Etor,E = [],[],[],[]
+    ir = IRFF_NP(atoms=images[0],
+                 libfile='ffield.json',
+                 nn='True')
+    ir.calculate(images[0])
+
+    ang_min = {}
+    ang_max = {}
+    for i,ang in enumerate(ir.angs):
+        t = tuple(ang)
+        ang_min[t] = ir.eang[i]
+        ang_max[t] = ir.eang[i]
+        
+    for i_,atoms in enumerate(images):       
+        ir.calculate(images[i_])
+        # print('%d Energies: ' %i_,'%12.4f ' %ir.E )
+
+        Eb.append(ir.Ebond)
+        Ea.append(ir.Eang)
+        Etor.append(ir.Etor)
+        E.append(ir.E)
+
+        for i,ang in enumerate(ir.angs):
+            t  = tuple(ang)
+            t_ = (ang[2],ang[1],ang[0])
+            if t in ang_min:
+               if ang_min[t]>ir.eang[i]:
+                  ang_min[t] = ir.eang[i]
+               if ang_max[t]<ir.eang[i]:
+                  ang_max[t] = ir.eang[i]
+            elif t_ in ang_min:
+               if ang_min[t_]>ir.eang[i]:
+                  ang_min[t_] = ir.eang[i]
+               if ang_max[t_]<ir.eang[i]:
+                  ang_max[t_] = ir.eang[i]
+            else:
+              ang_min[t] = ir.eang[i]
+              ang_max[t] = ir.eang[i]
+    ang_change = []
+    ang_name   = []
+    m_ = 0.0
+    mi = 0
+    for i,t in enumerate(ang_min):
+        m = ang_max[t] - ang_min[t]
+        if m>m_:
+           mi = i
+           m_ = m
+        ang_change.append(m)
+        ang_name.append(t)
+    ang  = ang_name[mi]
+    ang_ = '{:s}-{:s}-{:s}'.format(ir.atom_name[ang[0]],ir.atom_name[ang[1]],ir.atom_name[ang[2]])
+    print(ang_,ang,ang_change[mi]) 
+
 ## compare the total energy with DFT energy
 
 # images = Trajectory('md.traj')
