@@ -282,8 +282,8 @@ def write_qe_in(atoms,
        pwi.append('  cell_dynamics    = \'bfgs\' \n')
        pwi.append('/\n')  # terminate section
        pwi.append('\n')
-    else:
-       raise RuntimeError('-  This type of run of QE is not supported yet!')
+    #else:
+    #   raise RuntimeError('-  This type of run of QE is not supported yet!')
     
     if kspacing is not None:
        kgrid = kspacing_to_grid(atoms, kspacing)
@@ -328,8 +328,7 @@ def write_qe_in(atoms,
     # BandPath object or bandpath-as-dictionary:
     if 'K_POINTS' in kwargs:
        pwi.append('K_POINTS automatic\n')
-       pwi.append('{0[0]} {0[1]} {0[2]}  {1[0]:d} {1[1]:d} {1[2]:d}\n'
-                  ''.format(kwargs['K_POINTS'], koffset))
+       pwi.append('{:s} \n'.format(kwargs['K_POINTS']))
     else:
        pwi.append('K_POINTS automatic\n')
        pwi.append('{0[0]} {0[1]} {0[2]}  {1[0]:d} {1[1]:d} {1[2]:d}\n'
