@@ -149,7 +149,7 @@ def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
 def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                VariableCell='true',xcf='VDW',xca='DRSLL',basistype='DZP',
                tstep=200,us='F',P=0.0,
-               gen='poscar.gen',**kwargs):
+               gen='poscar.gen',FreeAtoms=None,**kwargs):
     if exists('siesta.MDE') or exists('siesta.MD_CAR'):
        system('rm siesta.MDE siesta.MD_CAR')
     if atoms is None:
@@ -158,7 +158,7 @@ def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                     VariableCell=VariableCell,
                     md=False,opt='CG',P=P,tstep=tstep,
                     xcf=xcf,xca=xca,basistype=basistype,
-                    us=us,**kwargs) # for MD
+                    us=us,FreeAtoms=FreeAtoms,**kwargs) # for MD
     run_siesta(ncpu=ncpu)
     images = siesta_traj(label=label)
     return images
@@ -183,7 +183,8 @@ def siesta_md(atoms=None,label='siesta',gen='poscar.gen',ncpu=4,supercell=[1,1,1
 def run_siesta(ncpu=12):
     system('mpirun -n %d siesta<in.fdf>siesta.out' %ncpu)
 
-def write_siesta_in(Atoms,coord='zmatrix',
+def write_siesta_in(Atoms,infdf='in.fdf',
+                    coord='zmatrix',
                     VariableCell='false',
                     fac=1.0,
                     opt='CG',  # 'FIRE' 'SinglePoint'
@@ -205,7 +206,7 @@ def write_siesta_in(Atoms,coord='zmatrix',
 
     cell = Atoms.get_cell()
     natm,atoms,X,table = get_neighbors(Atoms=Atoms,cell=cell,exception=['O-O','H-H'])
-    m = molecules(natm,atoms,X,table=table,cell=cell)
+    m = molecules(natm,atoms,X=X,table=table,cell=cell)
 
     m,A  = enlarge(m,cell=cell,fac = fac,supercell=[1,1,1])
     cell = A.get_cell()
@@ -234,7 +235,7 @@ def write_siesta_in(Atoms,coord='zmatrix',
     nspecie = len(species)
     natm    = len(atoms_label)
 
-    fdf = open('in.fdf','w')
+    fdf = open(infdf,'w')
     print('\n################## Species and atoms ##################',file=fdf)
     print('SystemName       siesta',file=fdf)
     print('SystemLabel      siesta',file=fdf)
@@ -875,7 +876,7 @@ def parse_out(outfile='siesta.out',spec={}):
     return atoms 
 
 def parse_fdf(label='siesta',spec={}):
-    fin = open(label+'.fdf','r') 
+    fin = open('{:s}.fdf'.format(label),'r') 
     lines= fin.readlines()
     fin.close()                     # getting informations from input file
     atom_name = []
@@ -892,10 +893,11 @@ def parse_fdf(label='siesta',spec={}):
         if ns and natom:
            break
 
-    positions = []
-    atom_name = []
-    cell      = []
-
+    positions  = []
+    atom_name  = []
+    cell       = []
+    fractional = False
+    
     for i,line in enumerate(lines):
         l = line.split()
         if len(l)>0:
@@ -924,8 +926,14 @@ def parse_fdf(label='siesta',spec={}):
                  unit = 0.529177249
               else:
                  unit = 1.0
+              
+           elif l[0]=='AtomicCoordinatesFormat':
+              if line.find('Fractional')>0 or line.find('fractional')>0:
+                 fractional = True
+                 
     cell      = np.array(cell)*unit
-    positions = np.dot(positions,cell)
+    if fractional:
+       positions = np.dot(positions,cell)
     A = Atoms(atom_name,positions,cell=cell,pbc=[True,True,True])
     # A.write(label+'.gen')
     return A
