@@ -6,6 +6,7 @@ from ase import Atoms
 from ase.io import read  # ,write
 from ase.io.trajectory import TrajectoryWriter
 from ase.calculators.singlepoint import SinglePointCalculator
+from ase.cell import Cell
 import matplotlib.pyplot as plt
 from ..molecule import press_mol
 from ..reaxfflib import write_lib
@@ -25,13 +26,12 @@ GULP calculation;
 end;
 
 library reax;
-output movie xyz his.xyz;
+output movie arc his;
 dump 100 restart.grs;
 
 '''
 
 def write_gulp_in(A, runword='gradient qiterative nosymmetry conv verb debu',
-                  T=298.0,
                   time_step=0.1,
                   tot_step=10.0,
                   maxcyc=2000,
@@ -55,7 +55,7 @@ def write_gulp_in(A, runword='gradient qiterative nosymmetry conv verb debu',
     elif rw[0] == 'md':
         #  if rw[1]=='conv':
         #     print('ensemble nvt',file=finp)
-        #     print('tau_thermostat     0.05 ps',file=finp)
+        #     print('tau_thermostat     0.1 ps',file=finp)
         #  if rw[1]=='conp':
         #     print('integrator leapfrog verlet',file=finp)
         #     print('ensemble           npt',file=finp)
@@ -63,20 +63,28 @@ def write_gulp_in(A, runword='gradient qiterative nosymmetry conv verb debu',
         #     print('tau_barostat       0.1 ps',file=finp)
         #     print('tau_thermostat     0.1 ps',file=finp)
         if 'ensemble' in kwargs:
-            print('ensemble   =  {:s} '.format(kwargs['ensemble']), file=finp)
+            print('ensemble          {:s} '.format(kwargs['ensemble']), file=finp)
         if 'tau_thermostat' in kwargs:
-            print('tau_thermostat   =  {:f} ps'.format(kwargs['tau_thermostat']), file=finp)
+            print('tau_thermostat    {:f} ps'.format(kwargs['tau_thermostat']), file=finp)
 
-        print('temperature        %f K' % T, file=finp)
+        # print('temperature        %f K' % T, file=finp)
         print('timestep           %f fs' % time_step, file=finp)
         print('production         %f ps' %
               float(tot_step*time_step/1000.0), file=finp)
         print('equilibration      0.0 ps', file=finp)
-        print('write              1', file=finp)
-        print('sample             1', file=finp)
-
+        if 'write_MD' in kwargs:
+            print('write_MD           {:d}'.format(kwargs['write_MD']), file=finp)
+        else:
+            print('write_MD           1  ', file=finp)
+        if 'sample' in kwargs:
+            print('sample             {:d}'.format(kwargs['sample']), file=finp)
+        else:
+            print('sample             1', file=finp)
+        
+    if 'T' in kwargs: 
+       print('temperature          {:f} K'.format(kwargs['T']),file=finp)
     if 'pressure' in kwargs:
-       print('pressure          {:f} GPa'.format(kwargs['pressure']),file=finp)
+       print('pressure             {:f} GPa'.format(kwargs['pressure']),file=finp)
 
     print('#', file=finp)
     print('title', file=finp)
@@ -84,15 +92,17 @@ def write_gulp_in(A, runword='gradient qiterative nosymmetry conv verb debu',
     print('end', file=finp)
     print('#', file=finp)
     if lib == 'brenner':
-        print('%s' % lib, file=finp)
+        print('{:s}'.format(lib), file=finp)
     else:
         print('library %s' % lib, file=finp)
 
     if output is None:
-        print('output movie {:d} xyz his.xyz'.format(
-            movieFrequency), file=finp)
+       if rw[0] == 'md':
+          print('output movie xyz his.xyz', file=finp)
+       else:
+          print('output movie arc his.arc', file=finp)
     else:
-        print('output {:s}'.format(output), file=finp)
+       print('output {:s}'.format(output), file=finp)
     print('output cif gulp produces gulp.cif', file=finp)
     print('dump {:d} restart.grs'.format(restart), file=finp)
     print('#', file=finp)
@@ -110,7 +120,6 @@ def write_gulp_in(A, runword='gradient qiterative nosymmetry conv verb debu',
         print(sp, 'core', x[0], x[1], x[2], 0.0, 1.0, 0.0, 1, 1, 1, file=finp)
     finp.close()
 
-
 def get_reaxff_q(natom, fo='out'):
     q = []
     fout = open(fo, 'r')
@@ -133,9 +142,22 @@ def get_reaxff_q(natom, fo='out'):
         q.append(float(l[2]))
     return q, ecoul, eself, evdw
 
-
-def get_reax_energy(fo='out'):
+def get_reax_energy(fo='out',e_kw='ReaxFF force field'):
     fout = open(fo, 'r')
+    e=0.0
+    ebond=0.0
+    elp=0.0
+    eover=0.0
+    eunder=0.0 
+    eang=0.0 
+    epen=0.0 
+    tconj=0.0 
+    etor=0.0 
+    fconj=0.0 
+    evdw=0.0 
+    ehb=0.0 
+    ecl=0.0 
+    esl = 0.0
     for line in fout.readlines():
         if line.find('E(bond)') >= 0:
             ebond = float(line.split()[2])
@@ -163,8 +185,8 @@ def get_reax_energy(fo='out'):
             ecl = float(line.split()[2])
         elif line.find('E(self)') >= 0:
             esl = float(line.split()[2])
-        elif line.find('ReaxFF force field') >= 0:
-            e = float(line.split()[4])
+        elif line.find(e_kw) >= 0:
+            e = float(line.split()[-2])
     fout.close()
     return e, ebond, elp, eover, eunder, eang, epen, tconj, etor, fconj, evdw, ehb, ecl, esl
 
@@ -220,7 +242,6 @@ def plot_md(E, Epot, T, P, show=True):
         plt.savefig('MD_results.pdf')
     plt.close()
 
-
 def plot_optimize(log='gulp.out'):
     ''' plot the energies or other from gulp log file'''
     flog = open(log, 'r')
@@ -243,7 +264,6 @@ def plot_optimize(log='gulp.out'):
     plt.savefig('energy_vs_iteration.png')
     plt.close()
 
-
 def get_lattice(inp='inp-gulp'):
     finp = open(inp, 'r')
     il = 0
@@ -262,7 +282,6 @@ def get_lattice(inp='inp-gulp'):
     finp.close()
     return np.array(cell)
 
-
 def reaxyz(fxyz):
     # cell = get_lattice()
     f = open(fxyz, 'r')
@@ -309,12 +328,49 @@ def xyztotraj(fxyz, inp='inp-gulp', checkMol=False, mode='w', traj='gulp.traj',
         if checkMol:
             A = press_mol(A)
 
-        calc = SinglePointCalculator(A, energy=e[i])
-        A.set_calculator(calc)
+        A.calc = SinglePointCalculator(A, energy=e[i])
         his.write(atoms=A)
         del A
     his.close()
 
+def arctotraj(arc,mode='w',checkMol=False,traj='gulp.traj'):
+    his = TrajectoryWriter(traj, mode=mode)
+
+    with open(arc,'r') as f:
+        lines = f.readlines()
+
+    atom_read_start = False
+    natom           = 0
+    pos             = []
+    atom_name       = []
+    e               = 0.0
+    cell            = [10.0,10.0,10.0]
+    for line in lines:   #  get number of atoms
+        l = line.split()
+        if line.find('PBC')>=0 and len(l)>=7:
+           atom_read_start = True
+           cell = Cell.fromcellpar([float(l[1]),float(l[2]),float(l[3]), 
+                                    float(l[4]),float(l[5]),float(l[6])])
+        elif line.find('end')>=0 :
+           atom_read_start = False
+           if pos:
+              A = Atoms(atom_name, pos, cell=cell, pbc=[True, True, True])
+              if checkMol:
+                 A = press_mol(A)
+              A.calc = SinglePointCalculator(A, energy=e)
+              his.write(atoms=A)
+              del A
+           pos       = []
+           atom_name = []
+        elif line.find('GULP calculation')>=0 :
+           e = float(l[2])/23.0604
+        if atom_read_start and l[0]!='PBC' and len(l)==9:
+           natom += 1
+           pos.append([float(l[1]),float(l[2]),float(l[3])])
+           atom_name.append(l[0])
+
+    his.close()
+
 def opt(atoms=None,T=350,gen='siesta.traj',step=200,i=-1,l=0,
         pressure=0.0,
         lib='reaxff_nn',ncpu=1):
@@ -330,7 +386,7 @@ def opt(atoms=None,T=350,gen='siesta.traj',step=200,i=-1,l=0,
                   lib=lib)
     # print('\n-  running gulp optimize ...')
     if ncpu > 1:
-        system('mpirun -n %d gulp<inp-gulp>gulp.out' % ncpu)
+        system('mpirun -n {:d} gulp<inp-gulp>gulp.out'.format(ncpu))
     else:
         system('gulp<inp-gulp>gulp.out')
 
@@ -340,6 +396,8 @@ def opt(atoms=None,T=350,gen='siesta.traj',step=200,i=-1,l=0,
          for line in fo:
              if line.find('Final energy')>=0:
                 e = float(line.split()[3])
+             elif line.find('Final enthalpy')>=0:
+                e = float(line.split()[3])
     return e,atoms
 
 def nvt(atoms=None, gen='poscar.gen', T=350, time_step=0.1, tot_step=100,
@@ -499,7 +557,7 @@ def get_gulp_force_from_drv(fdrv, wforce=False):
     return np.array(forces)
 
 
-def get_gulp_forces(images, traj='gulp_force.traj', ffield='reaxff_nn', wforce=False):
+def get_gulp_forces(images, traj='gulp_force.traj', ffield='reaxff_nn',e_kw='ReaxFF force field', wforce=False):
     ''' calculate the force by GULP '''
     his = TrajectoryWriter(traj, mode='w')
     for atoms in images:
@@ -510,12 +568,10 @@ def get_gulp_forces(images, traj='gulp_force.traj', ffield='reaxff_nn', wforce=F
         forces = get_gulp_force_from_drv('gulp.drv', wforce=wforce)
         #atoms.forces = forces
         (e, eb_, el_, eo_, eu_, ea_, ep_,
-         etc_, et_, ef_, ev_, ehb_, ecl_, esl_) = get_reax_energy(fo='out')
+         etc_, et_, ef_, ev_, ehb_, ecl_, esl_) = get_reax_energy(fo='out',e_kw=e_kw)
         #e = atoms.get_potential_energy()
-        calc = SinglePointCalculator(atoms, energy=e,
-                                     forces=forces)
-
-        atoms.set_calculator(calc)
+        atoms.calc = SinglePointCalculator(atoms, energy=e,forces=forces)
+        # atoms.set_calculator(calc)
         his.write(atoms=atoms)
     his.close()
     return atoms
