@@ -9,11 +9,10 @@ import numpy as np
 from ase.optimize import BFGS
 from ase.md.velocitydistribution import MaxwellBoltzmannDistribution
 from ase.md.verlet import VelocityVerlet
-from ase.io.trajectory import Trajectory
+from ase.io.trajectory import Trajectory,TrajectoryWriter
 from ase import units
 
 
-
 def get_pressure(atoms):           
     ir = IRFF(atoms=atoms_md[-1],
             libfile='ffield.json',
@@ -63,7 +62,8 @@ def getBonds(natom,r,rcut):
 class IRMD(object):
   ''' Intelligent Reactive Molecular Dynamics '''
   def __init__(self,label=None,atoms=None,gen='poscar.gen',ffield='ffield.json',
-               index=-1,totstep=100,vdwnn=False,nn=True,
+               index=-1,totstep=100,vdwnn=False,nn=True,nomb=False,
+               active=False,period=30,uncertainty=0.96,
                initT=300,Tmax=10000,time_step=0.1,Iter=0,
                ro=None,rmin=0.6,rmax=1.3,angmax=20.0,
                CheckZmat=False,zmat_id=None,zmat_index=None,InitZmat=None,
@@ -97,12 +97,16 @@ class IRMD(object):
       self.beta      = beta
       self.freeatoms = freeatoms
       self.print_interval = print_interval
+      self.active    = active        # an active leaning protocal for metal
+      self.uncertainty= uncertainty  # the limit of the uncertainty of bond length dirtribution
+      self.images    = []
+      self.rs        = []
 
       if self.atoms is None:
          self.atoms  = read(gen,index=index)
       
       self.atoms.calc= IRFF(atoms=self.atoms, mol=label,libfile=ffield,
-                            rcut=None,nn=nn,vdwnn=vdwnn)
+                            nomb=nomb,rcut=None,nn=nn,vdwnn=vdwnn)
       self.natom     = len(self.atoms)
       self.re        = self.atoms.calc.re
       self.dyn       = None
@@ -123,7 +127,10 @@ class IRMD(object):
 
   def run(self):
       self.zv,self.zvlo,self.zvhi = None,0.0,0.0
-      self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs,trajectory='md.traj') 
+      if self.active:
+         self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs)
+      else:
+         self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs,trajectory='md.traj') 
       if (not self.learnpair is None) and (not self.beta is None):
          vij = self.atoms.positions[self.learnpair[1]] - self.atoms.positions[self.learnpair[0]]
          rij = np.sqrt(np.sum(np.square(vij)))
@@ -207,6 +214,31 @@ class IRMD(object):
                             bonds      = getBonds(self.natom,self.atoms.calc.r,1.17*self.re)
                             newbond    = self.checkBond(bonds)
              self.zmats.append(zmat)
+          elif self.active:
+             self.images.append(a.copy())
+             bo0   =  a.calc.bo0.detach().numpy()
+             r     = a.calc.r.detach().numpy()
+             mask  = np.where(bo0>=0.0001,1,0)     # 掩码，用于过虑掉非成键键长
+            
+             
+             r_ = self.r*mask
+             r_ = r_[r_!=0]
+             self.rs.append(np.var(r_))
+             
+             if self.step%self.period==0:
+                self.rs.append(np.mean(r_))
+                try:
+                   assert self.uncertainty <= self.rs[-1], 'The varience is bigger than limit.'
+                except:
+                   print('The varience is bigger than limit, stop at %d.' %self.step)
+                   self.dyn.max_steps = self.dyn.nsteps-1
+               # n           = np.sum(mask)
+               # if np.mean(self.rs) > 0.01:
+               #    print(self.images)
+               #    break
+                r_          = []
+                self.images = []
+                self.rs     = []
           else:
              r    = a.calc.r.detach().numpy()
              i_   = np.where(np.logical_and(r<self.rmin*self.ro,r>0.0001))
@@ -316,6 +348,11 @@ class IRMD(object):
   
       self.dyn.attach(check,interval=1)
       self.dyn.run(0.00001,self.totstep)
+      if active:
+         traj  = TrajectoryWriter('md.traj',mode='w')
+         for atoms in self.images:
+             traj.write(atoms=atoms)
+         traj.close()
       return self.Deformed,self.zmats,self.zv,self.zvlo,self.zvhi
 
   def checkBond(self,bonds):
