@@ -1,11 +1,13 @@
+import random
 import numpy as np
 import types
+from sklearn.cluster import KMeans
 
 
 class Evolution:
     def __init__(self, func, n_dim, F=0.5,
                  size_pop=50, max_iter=200, prob_mut=0.5,
-                 X_input=None,scale=None,
+                 X_input=None,scale=None,n_clusters=1,
                  constraint_eq=tuple(), constraint_ueq=tuple()):
         self.func = func # func_transformer(func)
         assert size_pop % 2 == 0, 'size_pop must be even integer'
@@ -41,6 +43,7 @@ class Evolution:
         self.V, self.U = None, None
         # self.lb, self.ub = np.array(lb) * np.ones(self.n_dim), np.array(ub) * np.ones(self.n_dim)
         self.scale = scale
+        self.n_clusters = n_clusters
         self.crtbp()
 
     def crtbp(self):
@@ -48,7 +51,8 @@ class Evolution:
           创建初始                  族群: self.X     
           create the inital population: self.X
         '''
-        len_    = len(self.X_input)
+        len_       = len(self.X_input)
+        n_clusters = self.n_clusters if len_>self.n_clusters else len_
         pop_       = self.size_pop - len_
         self.X     = self.X_input
         Y_         = self.x2y()
@@ -61,14 +65,33 @@ class Evolution:
         if pop_>0:
            # X_    = np.random.uniform(low=self.lb, high=self.ub, size=(pop_, self.n_dim))  
            # Using a Gaussian Distribution instead of uniform distrution 使用高斯分布代替均匀分布
-           X_      = np.random.normal(loc=X_template, scale=self.scale, size=(pop_, self.n_dim))
-           merge_  = np.vstack([self.X_input,X_])
-           self.X  = merge_
+           XS = [self.X_input]
+           size_ = int(pop_/n_clusters)
+
+           if n_clusters>1:
+              random.seed()
+              clusters = {}
+              kmeans = KMeans(n_clusters=n_clusters, random_state=random.randint(0,10)).fit(self.X)
+              for i,l in enumerate(kmeans.labels_):
+                  if l not in clusters:
+                     clusters[l] = i
+              
+              for i,x in enumerate(clusters):
+                  i_ = clusters[x]
+                  size   = size_  if i != (n_clusters-1) else pop_-size_*i
+                  X_     = np.random.normal(loc=self.X[i_], scale=self.scale, size=(size, self.n_dim))
+                  XS.append(X_)
+           else:
+              X_  = np.random.normal(loc=X_template, scale=self.scale, size=(pop_, self.n_dim))
+              XS.append(X_)
+           self.X  = np.vstack(XS)
+
         elif pop_==0:
            self.X = self.X_input
         else:
            raise RuntimeError('The current population is larger than max defination!')
         # print('The length of current population:',len(self.X))
+        assert len(self.X)==self.size_pop,'Error: the length of the population is not equal to it set value!'
         return self.X
 
     def register(self, operator_name, operator, *args, **kwargs):
@@ -165,8 +188,8 @@ class Evolution:
 
         # print('\n----------------------------------------------------------------',file=logfile)
         # print('',file=logfile)
-        global_best_index = np.array(self.generation_best_Y).argmin()
-        global_best_X = self.generation_best_X[global_best_index]
+        self.global_best_index = np.array(self.generation_best_Y).argsort()
+        global_best_X = self.generation_best_X[self.global_best_index[0]]
         global_best_Y = self.func(np.array([global_best_X]))
         return global_best_X, global_best_Y
 
