@@ -13,7 +13,7 @@ import matplotlib.pyplot as plt
 from scipy.optimize import minimize_scalar
 from irff.ml.data import get_data
 from irff.data.ColData import ColData
-from irff.initCheck import init_bonds
+from irff.intCheck import init_bonds
 tf.compat.v1.disable_eager_execution()
 
 
@@ -25,13 +25,16 @@ def resolve():
 
 
 class Linear_be(object):
-    def __init__(self,Bp,D,B,E,bonds=None):
+    def __init__(self,Bp,D,B,E,be_layer=None,bonds=None,random_init=0):
         with open('ffield.json','r') as lf:
             self.j = js.load(lf)
         self.spec,bonds_,offd,angs,torp,hbs = init_bonds(self.j['p'])
         self.bonds = bonds_ if bonds is None else bonds 
         self.m = {}
-        hidelayer  = self.j['be_layer'][1]
+        hidelayer  = self.j['be_layer'][1] if be_layer is None else be_layer[1]
+        self.be_layer = self.j['be_layer'] if be_layer is None else be_layer
+
+
         self.E,self.B = {},{}
         for bd in self.bonds:
             self.m['fewi_'+bd] = tf.Variable(self.j['m']['fewi_'+bd],name='fewi_'+bd)
@@ -41,9 +44,20 @@ class Linear_be(object):
             self.m['few_'+bd]  = []
             self.m['feb_'+bd]  = []
             for i in range(hidelayer):
-                self.m['few_'+bd].append(tf.Variable(self.j['m']['few_'+bd][i],name='fewh_'+bd))
-                self.m['feb_'+bd].append(tf.Variable(self.j['m']['feb_'+bd][i],name='febh_'+bd))
-         
+                # w = np.array(self.j['m']['few_'+bd][i])
+                # b = np.array(self.j['m']['few_'+bd][i])
+                # m,n = w.shape
+                # print('hidden layer:',i,m,n)
+                if i+1 > self.j['be_layer'][1]:
+                   self.m['few_'+bd].append(tf.Variable(self.j['m']['few_'+bd][-1],name='fewh_'+bd))
+                   self.m['feb_'+bd].append(tf.Variable(self.j['m']['feb_'+bd][-1],name='febh_'+bd))
+                   self.j['m']['few_'+bd].append(self.j['m']['few_'+bd][-1])
+                   self.j['m']['feb_'+bd].append(self.j['m']['feb_'+bd][-1])
+                else:
+                   self.m['few_'+bd].append(tf.Variable(self.j['m']['few_'+bd][i],name='fewh_'+bd))
+                   self.m['feb_'+bd].append(tf.Variable(self.j['m']['feb_'+bd][i],name='febh_'+bd))
+        if hidelayer > self.j['be_layer'][1]:
+           self.j['be_layer'][1] = hidelayer
         for bd in self.bonds:
             self.E[bd] = tf.compat.v1.placeholder(tf.float32,shape=[None,1],name='E_%s' %bd)
             self.B[bd] = tf.compat.v1.placeholder(tf.float32,shape=[None,3],name='B_%s' %bd)
@@ -54,17 +68,23 @@ class Linear_be(object):
     def build_graph(self):
         #print('build graph ...')
         loss = 0.0
+        self.E_pred = {}
         for bd in self.bonds:
             ai   = tf.sigmoid(tf.matmul(self.B[bd],self.m['fewi_'+bd])  + self.m['febi_'+bd])
-            if self.j['be_layer'][1]>0:
-               ah = tf.sigmoid(tf.matmul(ai,self.m['few_'+bd][0]) + self.m['feb_'+bd][0])
+            if self.be_layer[1]>0:
+               for i in range(self.be_layer[1]):
+                   if i==0:
+                      a_ = ai
+                   else:
+                      a_ = ah
+                   ah = tf.sigmoid(tf.matmul(a_,self.m['few_'+bd][i]) + self.m['feb_'+bd][i])
                ao = tf.sigmoid(tf.matmul(ah,self.m['fewo_'+bd]) + self.m['febo_'+bd])
             else:
                ao = tf.sigmoid(tf.matmul(ai,self.m['fewo_'+bd]) + self.m['febo_'+bd])
 
-            e_pred = ao
-            # loss+= tf.sqrt(tf.reduce_sum(tf.square((e-e_pred)*self.j['p']['Desi_'+bd]*4.3364432032e-2)))
-            loss  += tf.nn.l2_loss((self.E[bd]-e_pred)*self.j['p']['Desi_'+bd]*4.3364432032e-2)
+            self.E_pred[bd] = ao
+            # loss+= tf.reduce_sum(tf.square(self.E[bd]-e_pred))
+            loss  += tf.nn.l2_loss(self.E[bd]-self.E_pred[bd])
         return loss
 
     def session(self,learning_rate=3.0e-4,method='AdamOptimizer'):
@@ -102,7 +122,7 @@ class Linear_be(object):
             self.j['m']['fewo_'+bd] = self.sess.run(self.m['fewo_'+bd]).tolist()
             self.j['m']['febo_'+bd] = self.sess.run(self.m['febo_'+bd]).tolist()
 
-            for i in range(self.j['be_layer'][1]):
+            for i in range(self.be_layer[1]):
                 self.j['m']['few_'+bd][i] = self.sess.run(self.m['few_'+bd][i]).tolist()
                 self.j['m']['feb_'+bd][i] = self.sess.run(self.m['feb_'+bd][i]).tolist()
         
@@ -127,6 +147,7 @@ class Linear_bo(object):
             for i in range(self.j['mf_layer'][1]):
                 self.m['fmw_'+sp].append(tf.Variable(self.j['m']['fmw_'+sp][i],name='fmwh_'+sp))
                 self.m['fmb_'+sp].append(tf.Variable(self.j['m']['fmb_'+sp][i],name='fmbh_'+sp))
+                
         for bd in self.bonds:
             self.D[bd]   = tf.compat.v1.placeholder(tf.float32,shape=[None,3],name='D_%s' %bd)
             self.D_t[bd] = tf.compat.v1.placeholder(tf.float32,shape=[None,3],name='Dt_%s' %bd)
@@ -142,16 +163,29 @@ class Linear_bo(object):
         for bd in self.bonds:
             atomi,atomj = bd.split('-')
             ai   = tf.sigmoid(tf.matmul(self.D[bd],self.m['fmwi_'+atomi])  + self.m['fmbi_'+atomi])
-            ah   = tf.sigmoid(tf.matmul(ai,self.m['fmw_'+atomi][0]) + self.m['fmb_'+atomi][0])
+            for i in range(self.j['mf_layer'][1]):
+                if i==0:
+                   a_ = ai
+                else:
+                   a_ = ah
+                ah   = tf.sigmoid(tf.matmul(a_,self.m['fmw_'+atomi][i]) + self.m['fmb_'+atomi][i])
+                
             ao   = tf.sigmoid(tf.matmul(ah,self.m['fmwo_'+atomi]) + self.m['fmbo_'+atomi])
 
             ai_t = tf.sigmoid(tf.matmul(self.D_t[bd],self.m['fmwi_'+atomj]) + self.m['fmbi_'+atomj])
-            ah_t = tf.sigmoid(tf.matmul(ai_t,self.m['fmw_'+atomj][0]) + self.m['fmb_'+atomj][0])
+            for i in range(self.j['mf_layer'][1]):
+                if i==0:
+                   a_ = ai_t
+                else:
+                   a_ = ah_t
+                ah_t  = tf.sigmoid(tf.matmul(a_,self.m['fmw_'+atomj][i]) + self.m['fmb_'+atomj][i])
+            #ah_t = tf.sigmoid(tf.matmul(ai_t,self.m['fmw_'+atomj][0]) + self.m['fmb_'+atomj][0])
             ao_t = tf.sigmoid(tf.matmul(ah_t,self.m['fmwo_'+atomj]) + self.m['fmbo_'+atomj])
 
             b_pred = self.Bp[bd]*ao*ao_t
-            # loss+= tf.sqrt(tf.reduce_sum(tf.square(b-b_pred)))
-            loss  += tf.nn.l2_loss(self.B[bd]-b_pred)
+            loss+= tf.sqrt(tf.reduce_sum(tf.square(self.B[bd]-b_pred)))
+            # loss+= tf.nn.l2_loss(self.B[bd]-b_pred)
+            # loss+= tf.compat.v1.losses.absolute_difference(self.B[bd]-b_pred)
         return loss
 
     def session(self,learning_rate=3.0-4,method='AdamOptimizer'):
