@@ -20,6 +20,7 @@ def train(step=5000,print_step=100,writelib=500,
             max_generation=500,
                   size_pop=500,
                   init_pop=10,
+                n_clusters=1,
                   prob_mut=0.5,
                  potential=None,
                    trainer=None,
@@ -27,6 +28,7 @@ def train(step=5000,print_step=100,writelib=500,
                      scale={},
             variable_scale=0,
              max_data_size=1000,
+                regenerate=False,
           end_search_ratio=0.999999,
             end_search_nan=False,
                GAThreshold=0.5):
@@ -37,7 +39,7 @@ def train(step=5000,print_step=100,writelib=500,
                  fcsv=fcsv,to_evaluate=to_evaluate,
                  step=evaluate_step,print_step=print_step,writelib=writelib,
                  evaluate_ffield=evaluate_ffield,pop=init_pop,scale=scale,
-                 parameters=parameters)
+                 n_clusters=n_clusters,parameters=parameters)
     d.sort_values(axis=0,by='score',ascending=False,inplace=True)
 
     X       = d.values[:, : -1]
@@ -45,7 +47,7 @@ def train(step=5000,print_step=100,writelib=500,
     columns = d.columns
     new_row = {}
     zrow    = d.index[0]
-    
+
     scale_  = []
     for col in columns:
         new_row[col] = [d.loc[zrow, col]]
@@ -54,7 +56,7 @@ def train(step=5000,print_step=100,writelib=500,
            if key in scale:
               scale_.append(scale[key])
            else:
-              scale_.append(0.01)
+              scale_.append(0.001)
 
     scale_  = np.array(scale_)
     new_row = pd.DataFrame(new_row)
@@ -80,6 +82,13 @@ def train(step=5000,print_step=100,writelib=500,
     do_gen    = True
     keep_best = 0
     while score<scoreConvergence and it_< max_ml_iter:
+        # if step>0 and it_>0:
+        #    d = evaluate(model=potential,trainer=trainer,
+        #                 fcsv=fcsv,to_evaluate=0.0,
+        #                 step=evaluate_step,
+        #                 evaluate_ffield=False,
+        #                 n_clusters=1,parameters=parameters)
+        #    d.sort_values(axis=0,by='score',ascending=False,inplace=True)
         size_ = d.shape[0]
         zrow  = d.index[0]
         sizepop = int(size_pop/2)
@@ -95,11 +104,11 @@ def train(step=5000,print_step=100,writelib=500,
         max_ = max(feature_importances)
 
         if variable_scale==2 and score_>0.98:
-           _scale = scale_*np.random.choice([0.1,1.0,10.0])*feature_importances*feature_importances/(max_*max_)
-        elif variable_scale>=1 and score_>0.94:
-           _scale = scale_*np.random.choice([0.1,1.0,10.0])*feature_importances/max_
+           _scale = scale_*feature_importances*feature_importances/(max_*max_) # *np.random.choice([0.1,1.0,10.0])
+        elif variable_scale>=1 and score_>0.96:
+           _scale = scale_*feature_importances/max_ # *np.random.choice([0.1,1.0,10.0])
         else:
-           _scale = scale_*np.random.choice([0.1,1.0,10.0])
+           _scale = scale_ # *np.random.choice([0.1,1.0,10.0])
 
         galog = open('evolution.log','a')
         print('\n           ----------------------------------',file=galog)
@@ -121,7 +130,7 @@ def train(step=5000,print_step=100,writelib=500,
            # lb=0.9*np.min(X_,axis=0)
            # ub=1.1*np.max(X_,axis=0)
            de = Evolution(func,n_dim=X_.shape[1], F=0.5,size_pop=size_pop,
-                          scale=_scale,max_iter=max_generation, 
+                          scale=_scale,max_iter=max_generation, n_clusters=n_clusters,
                           prob_mut=prob_mut,X_input=X_)    
 
            best_x,best_y = de.run(logfile=galog)                     ###   PSO GMM
@@ -135,18 +144,21 @@ def train(step=5000,print_step=100,writelib=500,
                best_x[i_] = x_ = float('{:.6f}'.format(x_))
                print('{:16s} {:9.6f} |'.format(cn,x_),end=' ',file=galog)
            print('\n--------------------------------------------------------------------------------------',file=galog)
-           keep_ = True
-           while keep_:
+           keep_ = True if step==0 else False
+           cycle = 0 
+           while keep_ and cycle<10:
                for i,key in enumerate(columns):
                    if key != 'score':
                       if abs(new_row.loc[0,key] - best_x[i])>=0.000001:
                          keep_ = False
                          print('                {:16s} {:9.6f} -> {:9.6f}'.format(key,new_row.loc[0,key],best_x[i]),file=galog)
                if keep_: 
-                  print(' The parameter vector keep best, a random parameter set is chosen ...',file=galog)
-                  i = np.random.choice(de.size_pop)
-                  best_x = de.X[i]
-                  best_y = de.Y[i]
+                  # print(' The parameter vector keep best, a random parameter set is chosen ...',file=galog)
+                  print(' The parameter vector keep best, a second best parameter set is chosen ...',file=galog)
+                  # i = np.random.choice(de.size_pop)
+                  cycle += 1
+                  best_x = de.X[de.global_best_index[cycle]]
+                  best_y = de.Y[de.global_best_index[cycle]]
         else:
            print('  The score of current parameters set looks good, need not do the genetic step.',file=galog)
         print('--------------------------------------------------------------------------------------',file=galog)
@@ -226,8 +238,9 @@ def train(step=5000,print_step=100,writelib=500,
            break
            
         d.to_csv(fcsv)
-        it_ += 1
         if it_>= max_ml_iter:
            print('\n  The maximum iterations have reached, the search is end.',file=galog)
         galog.close()
 
+        it_ += 1
+
