@@ -100,12 +100,13 @@ def select_mol(A=None,step=0,index=None,rcut=None,inbox=False):
     box = np.array([cel[0][0],cel[1][1],cel[2][2]])
     natm,atoms,X,table = get_neighbors(Atoms=A,r_cut=rcut,cell=box) #,exception=['O-O','H-H']
 
-    M = molecules(natm,atoms,X,
+    M = molecules(natm,atoms,
                   cell=box,
                   table=table,
+                  X=X,
                   check=True,inbox=inbox)
 
-    elems,x = [],[]
+    elems = [],[]
     ind     = []
     # flog    = open('log.txt','a')
     for mol in M:
@@ -117,7 +118,7 @@ def select_mol(A=None,step=0,index=None,rcut=None,inbox=False):
            # print(step,mol.mol_index,file=flog)
            # print(step,mol.atom_name,file=flog)
     # flog.close()
-    A = Atoms(elems,x,cell=cel,charges=ind,tags=ind)
+    A = Atoms(elems,cell=cel,charges=ind,tags=ind)
     return A
 
 
@@ -198,15 +199,17 @@ def packmol(strucs=[],supercell=[1,1,1],w=False,sizeiscell=True):
     return B         
 
 
-def press_mol(atoms,fac=1.0,inbox=False,check=True):
+def press_mol(atoms,fac=1.0,rcut=None,inbox=False,check=True):
     cell = atoms.get_cell()
     natm,atoms,X,table = get_neighbors(Atoms=atoms,
-                                r_cut=None,
+                                r_cut=rcut,
                                 cell=cell) #,exception=['O-O','H-H']
     
-    M = molecules(natm,atoms,X,
+    M = molecules(natm,atoms,
                   cell=cell,
                   table=table,
+                  X=X,
+                  rcut=rcut,
                   check=check,
                   inbox=inbox,
                   sizeiscell=True)
@@ -220,6 +223,7 @@ def SuperCell(m,cell=None,fac=1.0,supercell=None):
 
 
 def enlarge(m,cell=None,fac=1.0,supercell=None):
+    cell = cell*fac
     a = cell[0]
     b = cell[1]
     c = cell[2]
@@ -234,7 +238,9 @@ def enlarge(m,cell=None,fac=1.0,supercell=None):
     for mol in m:
         natm += mol.natom
 
-    elems,x = [None for na in range(natm)],[None for na in range(natm)]
+    elems = [None for na in range(natm)] 
+    x     = [None for na in range(natm)]
+    
     for mol in m:
         for i,na in enumerate(mol.mol_index):
             elems[na] = mol.atom_name[i]
@@ -256,9 +262,9 @@ def enlarge(m,cell=None,fac=1.0,supercell=None):
                           elems.extend(m[n_mol].atom_name)
                           x.extend(m_x)
  
-       a = [r*supercell[0]*fac for r in a]
-       b = [r*supercell[1]*fac for r in b]
-       c = [r*supercell[2]*fac for r in c]
+       a = [r*supercell[0] for r in a]
+       b = [r*supercell[1] for r in b]
+       c = [r*supercell[2] for r in c]
  
     A = Atoms(elems, x)
     A.set_cell([a,b,c])
@@ -284,18 +290,24 @@ def Molecules(atoms,rcut=None,check=False):
     natm,atoms,X,table = get_neighbors(Atoms=atoms,
                                        r_cut=rcut,
                                        cell=cell) #,exception=['O-O','H-H']
-    m = molecules(natm,atoms,X,
+    # for i,tab in enumerate(table):
+    #     print(i,tab)
+
+    m = molecules(natm,atoms,
                   cell=cell,
                   table=table,
+                  X=X,
+                  rcut=rcut,
                   check=check,
                   inbox=False,
                   sizeiscell=True)
     return m
 
 
-def molecules(natom,specs,X,cell=None,table=None,
-              check=False,
+def molecules(natom,specs,cell=None,table=None,
+              X=None,check=False,
               inbox=False,
+              rcut=None,
               sizeiscell=True):
     m = []
     for ID in range(natom):
@@ -314,7 +326,7 @@ def molecules(natom,specs,X,cell=None,table=None,
                mol_x.append(X[i])
 
            m.append(molecule(mol_index,atom_name,mol_x,
-                    check=check,inbox=inbox,
+                    check=check,inbox=inbox,rcut=rcut,
                     cell=cell,table=table,sizeiscell=sizeiscell))
     return m
 
@@ -327,16 +339,20 @@ class molecule(object):
                table=None,
                check=False,
                inbox=False,
-               rcut ={'Fe-C':2.0,'Fe-H':1.8,'Fe-N':2.0,'Fe-O':2.0,'Fe-Fe':2.2,
+               rcut=None):
+      self.inbox        = inbox
+      if rcut is None:
+         rcut =  {'Fe-C':2.0,'Fe-H':1.8,'Fe-N':2.0,'Fe-O':2.0,'Fe-Fe':2.2,
                        'C-C':1.8,'C-N':1.8,'C-O':1.8,'C-H':1.5,
                        'N-N':1.9,'N-H':1.8,'N-O':1.8,
                        'O-H':1.8,'O-O':1.7,
                        'H-H':1.3,
                        'Cl-C':2.0,'Cl-N':2.0,'Cl-O':2.0,'Cl-H':1.7,'Cl-Fe':2.2,
                        'F-C':1.85,'F-N':1.85,'F-O':1.85,'F-H':1.7,'F-Fe':1.8,'F-Cl':1.85,
-                       'other':1.8}):
-      self.inbox        = inbox
-      self.rcut         = rcut.copy()
+                       'others':1.8}
+         self.rcut = rcut.copy()
+      else:
+         self.rcut      = rcut.copy()
       self.mol_x        = np.array(mol_x)
       self.cell         = np.array(cell)
       self.sizeiscell   = sizeiscell
@@ -352,7 +368,7 @@ class molecule(object):
       self.center       = np.sum(self.mol_x,axis=0)/self.natom
 
       for bd in rcut:    # check rcut
-          if bd == 'other':
+          if bd == 'others':
              continue
           b = bd.split('-')
           bdr = b[1]+'-'+b[0]
@@ -447,9 +463,15 @@ class molecule(object):
               bd   = self.atom_name[iatom]+'-'+self.atom_name[jatom]
               vr   = self.mol_x[jatom] - self.mol_x[iatom]
               r_   = np.sqrt(np.sum(vr*vr,axis=0))
-              if r_>self.rcut[bd]: 
-                 self.mol_x[jatom] = self.mol_x[iatom] + self.vr[iatom][jatom]
+              r    = np.sqrt(np.sum(self.vr[iatom][jatom]*self.vr[iatom][jatom],axis=0))
 
+              if bd in self.rcut:
+                 rcut_ = self.rcut[bd] 
+              else:
+                 rcut_ = self.rcut['others']
+              if r_>rcut_: 
+                 self.mol_x[jatom] = self.mol_x[iatom] + self.vr[iatom][jatom]
+                 # print(iatom,jatom,r_,r)
 
   def move_atom(self,i,j):
       vr  = self.mol_x[j] - self.mol_x[i]
@@ -473,8 +495,8 @@ class molecule(object):
 if __name__ == '__main__':
    get_structure(struc='rdx',output='dftb',recover=True,center=True,supercell=[1,1,1])
    A = read('card.gen')
-   natm,atoms,X,table = get_neighbors(Atoms=A,r_cut=None,exception=['O-O','H-H'])
-   m = molecules(natm,atoms,X,table)
+   natm,atoms,table = get_neighbors(Atoms=A,r_cut=None,exception=['O-O','H-H'])
+   m = molecules(natm,atoms,table)
    cell = A.get_cell()
 
 
