@@ -124,7 +124,7 @@ class MPNN(ReaxFF):
                      ], # 
                nn=True,
                optmol=True,lambda_me=0.1,
-               opt=None,optword='nocoul',
+               opt=None,optword='nocoul',eaopt=[],
                mpopt=None,bdopt=None,mfopt=None,
                VariablesToOpt=None,
                batch_size=200,sample='uniform',
@@ -134,11 +134,11 @@ class MPNN(ReaxFF):
                belo={},
                vlo={'others':[(0.0,0.0)]},
                vup={'others':[(10.0,0.0)]},
-               pim={'others':10.0},
+               # pim={'others':10.0},
                spv_be=False,                             
-               spv_bo=None,                      # e.g. spv_bo={'C-C':(1.3,0.2,1.0)}
-               spv_pi=False,
-               spv_ang=False,
+               bo_clip=None,                      # e.g. bo_clip={'C-C':[(1.3,7.0,8.0,0,11,0.2,1.0)]}
+               pi_clip=False,                     # e.g. pi_clip={'C-C-C':[(7.5,8.5,0.8,1.8)]}
+               #spv_ang=False,                    
                spv_vdw=False,
                fixrcbo=False,
                weight={'others':1.0},
@@ -155,12 +155,12 @@ class MPNN(ReaxFF):
                vdwnn=False,VdwFunction=1,
                BOFunction=0,
                EnergyFunction=1,
-               MessageFunction=3,
+               MessageFunction=2,
                spec=[],
                lambda_bd=100000.0,
                lambda_pi=1.0,
                lambda_reg=0.01,
-               lambda_ang=1.0,
+               #lambda_ang=1.0,
                fluctuation=0.0,
                regularize_bo=True,
                regularize_be=True,
@@ -171,7 +171,7 @@ class MPNN(ReaxFF):
                maxstep=60000,
                emse=0.9,
                convergence=0.97,
-               lossConvergence=1000.0,
+               lossConvergence=0.0001,
                losFunc='n2',
                conf_vale=None,
                huber_d=30.0,
@@ -187,13 +187,13 @@ class MPNN(ReaxFF):
       self.EnergyFunction   = EnergyFunction
       self.MessageFunction  = MessageFunction
       self.BOFunction       = BOFunction
-      self.pim              = pim
-      self.spv_bo           = spv_bo
+      #self.pim              = pim
+      self.bo_clip          = bo_clip
+      self.pi_clip          = pi_clip
       self.spv_be           = spv_be
       self.beup             = beup
       self.belo             = belo
-      self.spv_pi           = spv_pi
-      self.spv_ang          = spv_ang
+      #self.spv_ang         = spv_ang
       self.spv_vdw          = spv_vdw
       self.vup              = vup
       self.vlo              = vlo
@@ -209,7 +209,7 @@ class MPNN(ReaxFF):
          self.regularize    = False
       self.lambda_reg       = lambda_reg
       self.lambda_pi        = lambda_pi
-      self.lambda_ang       = lambda_ang
+      #self.lambda_ang      = lambda_ang
       self.fluctuation      = fluctuation
       self.mf_layer         = mf_layer
       self.be_layer         = be_layer
@@ -226,7 +226,7 @@ class MPNN(ReaxFF):
       self.bdopt    = bdopt
       self.mfopt    = mfopt               # specify the element of message function to be optimized
       ReaxFF.__init__(self,libfile=libfile,dataset=dataset,
-                      dft=dft,atoms=atoms,cons=cons,opt=opt,optword=optword,
+                      dft=dft,atoms=atoms,cons=cons,opt=opt,optword=optword,eaopt=eaopt,
                       VariablesToOpt=VariablesToOpt,optmol=optmol,lambda_me=lambda_me,
                       batch_size=batch_size,sample=sample,
                       hbshort=hbshort,hblong=hblong,vdwcut=vdwcut,
@@ -280,7 +280,10 @@ class MPNN(ReaxFF):
           self.accur[mol] = 1.0 - tf.reduce_sum(input_tensor=tf.abs(self.E[mol]-self.dft_energy[mol]))/(sum_edft+0.00000001)
          
           self.Loss     += self.loss[mol]*w_
-          self.accuracy += self.accur[mol]
+          if mol.find('nomb')<0:
+             self.accuracy += self.accur[mol]
+          else:
+             self.nmol -= 1
 
       self.ME   = 0.0
       for mol in self.mols:
@@ -469,9 +472,13 @@ class MPNN(ReaxFF):
       elif self.MessageFunction==2:
          self.Dbi[bd]  = Di-h
          self.Dbj[bd]  = Dj-h
-         Fi   = fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],self.Dbj[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         # Fi   = fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],self.Dbj[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         #                      self.m,batch=self.batch,layer=self.mf_layer[1])
+         # Fj   = fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],self.Dbi[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         #                      self.m,batch=self.batch,layer=self.mf_layer[1])
+         Fi   = fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],h,self.Dbj[bd]],
                               self.m,batch=self.batch,layer=self.mf_layer[1])
-         Fj   = fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],self.Dbi[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         Fj   = fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],h,self.Dbi[bd]],
                               self.m,batch=self.batch,layer=self.mf_layer[1])
          F    = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
@@ -712,9 +719,13 @@ class MPNN(ReaxFF):
              k_ = k.split('_')[1]
              if k_ not in self.torp:
                 continue
-          self.p_[k] = float(p_[k])
-          if key in self.punit:
-             self.p_[k] = float(p_[k]/self.unit)
+          if k in self.ea_var:
+             self.p_[k] = self.ea_var[k]
+          else:
+             if key in self.punit:
+                self.p_[k] = float(p_[k]/self.unit)
+             else:
+                self.p_[k] = float(p_[k])
 
       score = loss if loss is None else -loss
          
@@ -825,7 +836,7 @@ class MPNN(ReaxFF):
       self.penalty_pi,self.penalty_ang = {},{}
       self.penalty_vdw = {} 
       self.penalty_be_cut,self.penalty_be = {},{}
-      self.penalty_s_bo,self.penalty_s = {},{}
+      self.penalty_s = {} #,{}
       self.penalty_rcut = {}
 
       for bd in self.bonds: 
@@ -908,20 +919,30 @@ class MPNN(ReaxFF):
              #    self.penalty_s[bd]  = tf.reduce_sum(input_tensor=self.bo0[bd]*fe_)
              #    penalty  = tf.add(self.penalty_s[bd]*self.lambda_bd,penalty)    ## penalty iterm for bond order
 
-             if self.spv_bo:
+             if self.bo_clip:
                 #fe_ = tf.where(tf.greater(self.S[bd],0.00001),1.0,0.0) 
                 #self.penalty_s[bd]  = tf.reduce_sum(input_tensor=self.bo0[bd]*fe_)
                 #penalty  = tf.add(self.penalty_s[bd]*self.lambda_bd,penalty)      ## penalty iterm for bond order
                 self.penalty_bo[bd] = tf.constant(0.0)
-                if (bd in self.spv_bo) or (bdr in self.spv_bo):
-                   bd_  = bd if bd in self.spv_bo else bdr
-                   r    = self.spv_bo[bd_][0]
-                   bo_l = self.spv_bo[bd_][1]
-                   bo_u = self.spv_bo[bd_][2]
-                   fe   = tf.where(tf.less_equal(self.rbd[bd],r),1.0,0.0) ##### r< r_e that bo > bore_
-                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((bo_l-self.bop[bd])*fe))
-                   fe   = tf.where(tf.greater_equal(self.rbd[bd],r),1.0,0.0) ##### r< r_e that bo > bore_
-                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((self.bop[bd]-bo_u)*fe))
+                if (bd in self.bo_clip) or (bdr in self.bo_clip):
+                   bd_  = bd if bd in self.bo_clip else bdr
+                   for sbo in self.bo_clip[bd_]:
+                       r,dil,diu,djl,dju,bo_l,bo_u = sbo
+                       fe   = tf.where(tf.logical_and(tf.less_equal(self.rbd[bd],r),
+                                                      tf.logical_and(tf.logical_and(tf.greater_equal(self.Dbi[bd],dil),
+                                                                                    tf.greater_equal(self.Dbj[bd],djl)), 
+                                                                     tf.logical_and(tf.less_equal(self.Dbi[bd],diu),
+                                                                                    tf.less_equal(self.Dbj[bd],dju))  ) ),
+                                       1.0,0.0)   ##### r< r_e that bo > bore_
+                       self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((bo_l-self.esi[bd])*fe)) 
+                                                                                    #     self.bo0[bd]
+                       fe   = tf.where(tf.logical_and(tf.greater_equal(self.rbd[bd],r),
+                                                      tf.logical_and(tf.logical_and(tf.greater_equal(self.Dbi[bd],dil),
+                                                                                    tf.greater_equal(self.Dbj[bd],djl)), 
+                                                                     tf.logical_and(tf.less_equal(self.Dbi[bd],diu),
+                                                                                    tf.less_equal(self.Dbj[bd],dju))  ) ),
+                                       1.0,0.0)  ##### r> r_e that bo < bore_
+                       self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd]-bo_u)*fe))
 
                 penalty  = tf.add(self.penalty_bo[bd]*self.lambda_bd,penalty) 
 
@@ -965,24 +986,20 @@ class MPNN(ReaxFF):
                        self.penalty_vdw[bd] = self.penalty_vdw[bd] + pen_v
                 penalty  = tf.add(self.penalty_vdw[bd]*self.lambda_bd,penalty)
 
-          # for sp in self.spec: 
-          #     pi_ = self.pim[sp] if sp in self.pim else self.pim['others']
-          #     if self.nsp[sp]>0:
-          #        self.penalty_pi[sp] = tf.reduce_sum(input_tensor=tf.nn.relu(self.Dpi[sp]-pi_))
-          #        penalty  = tf.add(self.penalty_pi[sp]*self.lambda_pi,penalty)
       if self.optword.find('noang')<0:
-         for ang in self.angs: 
-             if self.nang[ang]>0:
-                if self.spv_pi:
-                   pi_ = self.pim[ang] if ang in self.pim else self.pim['others']
-                   self.penalty_pi[ang] = tf.reduce_sum(input_tensor=tf.nn.relu(self.SBO[ang]-pi_))
+         if self.pi_clip:                        # regularize pi term
+            for ang in self.pi_clip: 
+                if self.nang[ang]>0:
+                   self.penalty_pi[ang] = tf.constant(0.0)
+                   for spi in self.pi_clip[ang]:
+                       Dl,Du,pil,piu = spi
+                       fpi = tf.where(tf.logical_and(tf.less_equal(self.D_p[ang],Du), 
+                                                    tf.greater_equal(self.D_p[ang],Dl)),
+                                       1.0,0.0)  
+                       self.penalty_pi[ang]+= tf.reduce_sum(input_tensor=tf.nn.relu((self.SBO[ang]-piu)*fpi))
+                       self.penalty_pi[ang]+= tf.reduce_sum(input_tensor=tf.nn.relu((pil-self.SBO[ang])*fpi))
                    penalty  = tf.add(self.penalty_pi[ang]*self.lambda_pi,penalty)
-                if self.spv_ang:
-                   self.penalty_ang[ang] = tf.reduce_sum(self.thet2[ang]*self.fijk[ang])
-                   penalty  = tf.add(self.penalty_ang[ang]*self.lambda_ang,penalty)
 
-         # self.penalty_pi = tf.reduce_sum(input_tensor=tf.nn.relu(self.DPI-2.0)) 
-         # penalty  = tf.add(self.penalty_pi*self.lambda_pi,penalty)
       if self.regularize:                              # regularize
          for sp in self.spec:
              for k in wb_message:
@@ -1004,11 +1021,11 @@ class MPNN(ReaxFF):
   def get_pentalty(self):
       (penalty_bop,penalty_bo,penalty_bo_rcut,
           penalty_be_cut,penalty_be,
-          penalty_s,penalty_s_bo,
+          penalty_s,
           penalty_rcut,rc_bo,
           penalty_vdw) = self.sess.run([self.penalty_bop,self.penalty_bo,self.penalty_bo_rcut,
                                          self.penalty_be_cut,self.penalty_be,
-                                         self.penalty_s,self.penalty_s_bo,self.penalty_rcut,self.rc_bo,
+                                         self.penalty_s,self.penalty_rcut,self.rc_bo,
                                          self.penalty_vdw],
                                         feed_dict=self.feed_dict)
       rcut = self.rcut
@@ -1030,8 +1047,8 @@ class MPNN(ReaxFF):
              print('Bond-Energy at radius cutoff penalty of           {:5s}: {:6.4f}'.format(bd,penalty_be_cut[bd]))
           if bd in penalty_be: 
              print('Bond-Energy fluctuation penalty of                {:5s}: {:6.4f}'.format(bd,penalty_be[bd]))
-          if bd in penalty_s_bo:
-             print('Penalty of Bond-Order should greater than zero of {:5s}: {:6.4f}'.format(bd,penalty_s_bo[bd]))
+          # if bd in penalty_s_bo:
+          #   print('Penalty of Bond-Order should greater than zero of {:5s}: {:6.4f}'.format(bd,penalty_s_bo[bd]))
           if bd in penalty_s:
              print('Anti-bond penalty of                              {:5s}: {:6.4f}'.format(bd,penalty_s[bd]))
           if bd in penalty_rcut:
@@ -1281,8 +1298,8 @@ def set_matrix(m_,spec,bonds,mfopt,mpopt,bdopt,messages,
         nin_  = 5
     elif MessageFunction==5 :
         nin_  = 3 
-    elif MessageFunction==2:
-        nin_  = 5 
+    # elif MessageFunction==2:
+    #     nin_  = 5 
     else:
         nin_  = 3
 
@@ -1311,14 +1328,14 @@ def set_matrix(m_,spec,bonds,mfopt,mpopt,bdopt,messages,
 
     nin_ = 1 if VdwFunction==1 else 3
 
-    # if vdwnn:
-    #    reuse_m = True if vdw_layer==vdw_layer_ and VdwFunction==VdwFunction_ else False
-    #    if not vdw_universal_nn is None:
-    #       set_universal_wb(m_=m_,pref='fv',bd=vdw_universal_nn[0],reuse_m=reuse_m,
-    #                         nin=nin_,nout=1,layer=vdw_layer,
-    #                         nnopt=mpopt[t+1],bias=0.867)
-    #    set_wb(m_=m_,pref='fv',reuse_m=reuse_m,nin=nin_,nout=1,layer=vdw_layer,
-    #                 vlist=bonds,nnopt=mpopt[-1],bias=0.867)
+    if  VdwFunction>0:
+       reuse_m = True if vdw_layer==vdw_layer_ and VdwFunction==VdwFunction_ else False
+       if not vdw_universal_nn is None:
+          set_universal_wb(m_=m_,pref='fv',bd=vdw_universal_nn[0],reuse_m=reuse_m,
+                            nin=nin_,nout=1,layer=vdw_layer,
+                            nnopt=mpopt[t+1],bias=0.867)
+       set_wb(m_=m_,pref='fv',reuse_m=reuse_m,nin=nin_,nout=1,layer=vdw_layer,
+                    vlist=bonds,nnopt=mpopt[-1],bias=0.867)
     return m
 
 def get_universal_nn(spec,bonds,bo_universal_nn,be_universal_nn,vdw_universal_nn,mf_universal_nn):
@@ -1370,4 +1387,36 @@ def get_universal_nn(spec,bonds,bo_universal_nn,be_universal_nn,vdw_universal_nn
            universal_nn.append('fm'+'_'+sp) # +str(t)
     return universal_nn
 
-                 
\ No newline at end of file
+'''
+Example:
+    rn = MPNN(libfile='ffield.json',
+              dataset=dataset,            
+              bo_clip={'C-H':[(1.8,9.5,11,2,11,0.0,0.0)],
+                      'H-O':[(1.3,2.0,11,2.78,11,0.0,0.0)],
+                      'C-C':[(1.9,7.9,11,7.9,11,0.0,0.0)] },
+              pi_clip={'N-C-N':[(7.8,8.7,1.65,1.8)],
+                      'C-C-C':[(7.8,8.7,1.61,1.8)],
+                      'C-C-H':[(8.0,8.8,1.61,1.8),(10.0,13,0,0.75)],
+                      'H-C-H':[(8.0,8.8,1.61,1.8)]},
+              weight=weight,
+              optword='nocoul',mpopt=mpopt,
+              opt=opt,cons=cons,clip=clip,
+              regularize_mf=1,regularize_be=1,regularize_bias=1,
+              lambda_reg=0.001,lambda_bd=100.0,lambda_me=0.0002,
+              mf_layer=[9,1],be_layer=[9,1],
+              EnergyFunction=1,MessageFunction=2,
+              mf_universal_nn=None,be_universal_nn=['C-H'],
+              messages=1,
+              bdopt=None,    # ['H-N'], 
+              mfopt=None,    # ['N'], 
+              batch_size=batch,
+              fixrcbo=False,
+              losFunc='n2',  # n2, mse, huber,abs
+              convergence=convergence) 
+
+    loss,accu,accMax,i,zpe =rn.run(learning_rate=lr,
+                      step=step,
+                      print_step=print_step,
+                      writelib=writelib,
+                      method='AdamOptimizer')  
+'''
