@@ -53,7 +53,7 @@ def get_pangle(p,atom_name,np_,p_ang,nang,angs):
             an = key+'_'+atom_name[ang[0]]+'-'+atom_name[ang[1]]+'-'+atom_name[ang[2]]
             if not an in p:
                an = key+'_'+atom_name[ang[2]]+'-'+atom_name[ang[1]]+'-'+atom_name[ang[0]]
-            P[key][a_] = p[an]
+            P[key][a_] = p[an] if an in p else 0.0
 
     P['val3'] = np.zeros([nang],dtype=np.float32)
     for a_ in range(nang):
