@@ -194,10 +194,24 @@ class reax_data(object):
       self.get_table()
       self.get_bonds(self.R)
 
-      self.compute_angle(self.R,self.vr)
-      self.compute_torsion(self.R,self.vr)
-
-      self.compute_hbond(image_rs)
+      if self.structure.find('nomb')>=0:
+         self.nhb  = 0
+         self.nang = 0
+         self.ntor = 0
+         self.hb_i,self.hb_j,self.hb_k = [],[],[]
+         self.abij,self.abjk = [],[]
+         self.ang_i,self.ang_j,self.ang_k = [],[],[]
+         self.tij,self.tjk,self.tkl = [],[],[]
+         self.tor_i,self.tor_j,self.tor_k,self.tor_l = [],[],[],[]
+         self.A,self.T,self.H,self.hij={},{},{},{}
+         self.na,self.nt={},{}
+         self.theta,self.s_ijk,self.s_jkl,self.w,self.rhb = None,None,None,None,None
+         self.frhb,self.hbthe = None,None
+      else:
+         self.compute_angle(self.R,self.vr)
+         self.compute_torsion(self.R,self.vr)
+         self.compute_hbond(image_rs)
+         
       self.compute_vdw(image_rs)
 
       # self.get_gulp_energy()
@@ -920,6 +934,7 @@ class reax_data(object):
       cell      = []
       forces    = []
       energy_nw = []
+      force_has_none = False
       for i,ind_ in enumerate(self.indexs):
           imag = images[ind_]
           if i==0:
@@ -937,13 +952,17 @@ class reax_data(object):
              force_ = imag.get_forces()
           except:
              force_ = None
+             force_has_none = True
              # print('-  ignoring the forces as are not available.')
           forces.append(force_)
           energy_nw.append(e)
       self.energy_nw = np.array(energy_nw)
       self.x         = np.array(x)
       self.cell      = np.array(cell)
-      self.forces    = np.array(forces) # ,dtype=np.float32)
+      if force_has_none:
+         self.forces = None
+      else:
+         self.forces = np.array(forces) # ,dtype=np.float32)
       # print(len(self.x),len(self.cell),len(self.forces))
 
   def set_parameters(self):
