@@ -37,10 +37,11 @@ class ReaxFF_nn(object):
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
                atol=0.001,
-               hbtol=0.001,
+               # hbtol=0.001,
                bore={'others':0.0},
                weight={'others':1.0},
                spv_vdw=False,vlo={'others':[(0.0,0.0)]},vup={'others':[(10.0,0.0)]},
+               bo_clip=None,                 # e.g. bo_clip={'C-C':(1.3,8.5,8.5,0.0,0.0)}
                interactive=False,
                ro_scale=0.1,
                clip_op=True,
@@ -52,7 +53,7 @@ class ReaxFF_nn(object):
                nnopt=True,
                be_universal_nn=None,be_layer=[9,0],
                mf_universal_nn=None,mf_layer=[9,0],
-               messages=1,
+               messages=1,MessageFunction=2,
                bo_layer=None,
                spec=[],
                board=False,
@@ -120,11 +121,13 @@ class ReaxFF_nn(object):
       self.lambda_ang    = lambda_ang
       self.mf_layer      = mf_layer
       self.be_layer      = be_layer
-      self.be_universal_nn= be_universal_nn
-      self.mf_universal_nn= mf_universal_nn
-      self.messages      = messages
+      self.be_universal_nn = be_universal_nn
+      self.mf_universal_nn = mf_universal_nn
+      self.messages        = messages
+      self.MessageFunction = MessageFunction
       self.spv_vdw       = spv_vdw
       self.spv_ang       = spv_ang
+      self.bo_clip       = bo_clip
       self.vlo           = vlo
       self.vup           = vup
       self.bo_layer      = bo_layer
@@ -151,15 +154,15 @@ class ReaxFF_nn(object):
       self.rcut,self.rcuta,self.re = self.read_lib()
       self.set_rcut(self.rcut,self.rcuta,self.re)
 
+      self.set_variable_list()
       self.ic = Intelligent_Check(re=self.re,clip=clip,spec=self.spec,bonds=self.bonds,
-                                  offd=self.offd,angs=self.angs,tors=self.torp)
+                                  offd=self.offd,angs=self.angs,tors=self.torp,ptor=self.p_tor)
       self.p_,self.m_ = self.ic.check(self.p_,self.m_,resetDeadNeuron=self.resetDeadNeuron)
 
       if not self.libfile.endswith('.json'):
          self.p_['acut']    = atol
-         self.p_['hbtol']   = hbtol
+         self.p_['hbtol']   = atol
 
-      self.set_neurons()
       self.torp          = self.checkTors(self.torp)
       
       self.lambda_bd     = lambda_bd
@@ -331,7 +334,8 @@ class ReaxFF_nn(object):
 
       self.Deltap,self.Delta,self.Bp = {},{},{}
       self.delta,self.Di,self.Dj,self.Di_boc,self.Dj_boc={},{},{},{},{}
-      self.D,self.H,self.Hsi,self.Hpi,self.Hpp = {},{},{},{},{}
+      self.D,self.D_si,self.D_pi,self.D_pp = {},{},{},{}
+      self.H,self.Hsi,self.Hpi,self.Hpp = {},{},{},{}
 
       self.SO,self.fbot,self.fhb = {},{},{}
       self.EBD,self.E = {},{}
@@ -341,12 +345,16 @@ class ReaxFF_nn(object):
       self.ebd   = {}
       self.rbd   = {}
       self.rbd_  = {}
+      self.Dbi   = {}
+      self.Dbj   = {}
       #self.bop_ = {}
       #self.bo0_ = {}
       for mol in self.mols:
           self.esi[mol]   = {}
           self.ebd[mol]   = {}
           self.rbd_[mol]  = {}
+          self.Dbi[mol]   = {}
+          self.Dbj[mol]   = {}
           #self.bop_[mol]  = {}
           #self.bo0_[mol]  = {}
 
@@ -429,7 +437,7 @@ class ReaxFF_nn(object):
                                             name='forces_%s' %mol)
 
   def build_graph(self):
-      print('-  building graph ...')
+      print('-  building graph: ')
       self.accuracy   = tf.constant(0.0,name='accuracy')
       self.accuracies = {}
       for mol in self.mols:
@@ -462,12 +470,22 @@ class ReaxFF_nn(object):
 
   def get_delta(self,mol):
       ''' compute the uncorrected Delta: the sum of BO '''
-      BOP = tf.zeros([1,self.batch[mol]])   # for ghost atom, the value is zero
+      BOP    = tf.zeros([1,self.batch[mol]])   # for ghost atom, the value is zero
+      BOP_si = tf.zeros([1,self.batch[mol]])   # for ghost atom, the value is zero
+      BOP_pi = tf.zeros([1,self.batch[mol]])   # for ghost atom, the value is zero
+      BOP_pp = tf.zeros([1,self.batch[mol]])   # for ghost atom, the value is zero
       self.get_bondorder_uc(mol)
+
       BOP              = tf.concat([BOP,self.bop[mol]],0)
-      # BOP            = tf.transpose(BOP,perm=[0,1])
+      BOP_si           = tf.concat([BOP,self.bop_si[mol]],0)
+      BOP_pi           = tf.concat([BOP,self.bop_pi[mol]],0)
+      BOP_pp           = tf.concat([BOP,self.bop_pp[mol]],0)
+
       self.Bp[mol]     = tf.gather_nd(BOP,self.blist[mol])
       self.Deltap[mol] = tf.reduce_sum(input_tensor=self.Bp[mol],axis=1,name='Deltap')
+      self.D_si[mol]   = [tf.reduce_sum(tf.gather_nd(BOP_si,self.blist[mol]),axis=1,name='Deltap_si')]
+      self.D_pi[mol]   = [tf.reduce_sum(tf.gather_nd(BOP_pi,self.blist[mol]),axis=1,name='Deltap_pi')]
+      self.D_pp[mol]   = [tf.reduce_sum(tf.gather_nd(BOP_pp,self.blist[mol]),axis=1,name='Deltap_pp')]
 
   def get_bond_energy(self,mol):
       self.get_delta(mol)
@@ -552,13 +570,30 @@ class ReaxFF_nn(object):
           hpp  = tf.slice(self.Hpp[mol][t-1],[b_[0],0],[b_[1],self.batch[mol]],name=bd+'_hpp_slice')
 
           b    = bd.split('-')
-          Dbi  = Di - h   
-          Dbj  = Dj - h   
-          
-          Fi   = fmessage(flabel,b[0],self.nbd[mol][bd],[Dbi,h,Dbj],self.m,
-                          batch=self.batch[mol],layer=self.mf_layer[1])
-          Fj   = fmessage(flabel,b[1],self.nbd[mol][bd],[Dbj,h,Dbi],self.m,
-                          batch=self.batch[mol],layer=self.mf_layer[1])
+
+          if self.MessageFunction==1:
+             Dsi_i = tf.gather_nd(self.D_si[mol][t-1],self.dilink[mol][bd]) - hsi
+             Dpi_i = tf.gather_nd(self.D_pi[mol][t-1],self.dilink[mol][bd]) - hpi
+             Dpp_i = tf.gather_nd(self.D_pp[mol][t-1],self.dilink[mol][bd]) - hpp 
+             
+             Dsi_j = tf.gather_nd(self.D_si[mol][t-1],self.djlink[mol][bd]) - hsi
+             Dpi_j = tf.gather_nd(self.D_pi[mol][t-1],self.djlink[mol][bd]) - hpi
+             Dpp_j = tf.gather_nd(self.D_pp[mol][t-1],self.djlink[mol][bd]) - hpp
+
+             Dpii  = Dpi_i + Dpp_i
+             Dpij  = Dpi_j + Dpp_j
+            
+             Fi    = fmessage(flabel,b[0],nbd_,[Dsi_i,Dpii,h,Dpij,Dsi_j],
+                              self.m,batch=self.batch[mol],layer=self.mf_layer[1])
+             Fj    = fmessage(flabel,b[1],nbd_,[Dsi_j,Dpij,h,Dpii,Dsi_i],
+                              self.m,batch=self.batch[mol],layer=self.mf_layer[1])
+          elif self.MessageFunction==2:
+             self.Dbi[mol][bd]  = Di - h   
+             self.Dbj[mol][bd]  = Dj - h   
+             Fi   = fmessage(flabel,b[0],nbd_,[self.Dbi[mol][bd],h,self.Dbj[mol][bd]],self.m,
+                             batch=self.batch[mol],layer=self.mf_layer[1])
+             Fj   = fmessage(flabel,b[1],nbd_,[self.Dbj[mol][bd],h,self.Dbi[mol][bd]],self.m,
+                             batch=self.batch[mol],layer=self.mf_layer[1])
           F    = Fi*Fj
           Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
 
@@ -574,15 +609,18 @@ class ReaxFF_nn(object):
 
   def message_passing(self,mol):
       ''' finding the final Bond-order with a message passing '''
-      self.H[mol]   = [self.bop[mol]]                     # 
-      self.Hsi[mol] = [self.bop_si[mol]]                  #
-      self.Hpi[mol] = [self.bop_pi[mol]]                  #
-      self.Hpp[mol] = [self.bop_pp[mol]]                  # 
-      self.D[mol]   = [self.Deltap[mol]]                  # get the initial hidden state H[0]
+      self.H[mol]    = [self.bop[mol]]                     # 
+      self.Hsi[mol]  = [self.bop_si[mol]]                  #
+      self.Hpi[mol]  = [self.bop_pi[mol]]                  #
+      self.Hpp[mol]  = [self.bop_pp[mol]]                  # 
+      self.D[mol]    = [self.Deltap[mol]]                  # get the initial hidden state H[0]
 
       for t in range(1,self.messages+1):
-          print('-  message passing for {:s} t={:d} ...'.format(mol,t))           
+          print('- {:s} '.format(mol))           
           BO    = tf.zeros([1,self.batch[mol]])           # for ghost atom, the value is zero
+          BOsi  = tf.zeros([1,self.batch[mol]])           # for ghost atom, the value is zero
+          BOpi  = tf.zeros([1,self.batch[mol]])           # for ghost atom, the value is zero
+          BOpp  = tf.zeros([1,self.batch[mol]])           # for ghost atom, the value is zero
           bo,bosi,bopi,bopp = self.get_bondorder(mol,t)
           self.H[mol].append(bo)                      # get the hidden state H[t]
           self.Hsi[mol].append(bosi)
@@ -590,9 +628,19 @@ class ReaxFF_nn(object):
           self.Hpp[mol].append(bopp)
 
           BO      = tf.concat([BO,bo],0)
-          D       = tf.gather_nd(BO,self.blist[mol])  
-          Delta   = tf.reduce_sum(input_tensor=D,axis=1)
+          BOsi    = tf.concat([BOsi,bosi],0)
+          BOpi    = tf.concat([BOpi,bopi],0)
+          BOpp    = tf.concat([BOpp,bopp],0)
+
+          Delta   = tf.reduce_sum(tf.gather_nd(BO,self.blist[mol]),axis=1) 
+          Dsi     = tf.reduce_sum(tf.gather_nd(BOsi,self.blist[mol]),axis=1) 
+          Dpi     = tf.reduce_sum(tf.gather_nd(BOpi,self.blist[mol]),axis=1) 
+          Dpp     = tf.reduce_sum(tf.gather_nd(BOpp,self.blist[mol]),axis=1)  
+
           self.D[mol].append(Delta)                  # degree matrix
+          self.D_si[mol].append(Dsi)
+          self.D_pi[mol].append(Dpi)
+          self.D_pp[mol].append(Dpp)
 
   def get_final_sate(self,mol):     
       self.Delta[mol]  = self.D[mol][-1]
@@ -686,8 +734,8 @@ class ReaxFF_nn(object):
       #self.so[atom]     = tf.gather_nd(self.SO,self.atomlist[atom])
       otrm1              = DIV_IF(1.0,self.Delta_lpcorr[mol]+val)
       # self.otrm2[atom] = tf.math.divide(1.0,1.0+tf.exp(self.p['ovun2_'+atom]*self.Delta_lpcorr[atom]))
-      otrm2             = tf.sigmoid(-ovun2*self.Delta_lpcorr[mol])
-      self.EOV[mol]     = self.SO[mol] *otrm1*self.Delta_lpcorr[mol]*otrm2
+      otrm2              = tf.sigmoid(-ovun2*self.Delta_lpcorr[mol])
+      self.EOV[mol]      = self.SO[mol] *otrm1*self.Delta_lpcorr[mol]*otrm2
 
   def get_eunder(self,mol,ovun2,ovun5):
       expeu1            = tf.exp(self.p['ovun6']*self.Delta_lpcorr[mol])
@@ -1006,7 +1054,10 @@ class ReaxFF_nn(object):
           self.accur[mol] = 1.0 - tf.reduce_sum(input_tensor=tf.abs(self.E[mol]-self.dft_energy[mol]))/(sum_edft+0.00000001)
          
           self.Loss     += self.loss[mol]*w_
-          self.accuracy += self.accur[mol]
+          if mol.find('nomb')<0:
+             self.accuracy += self.accur[mol]
+          else:
+             self.nmol -= 1
 
       self.ME   = 0.0
       for mol in self.mols:
@@ -1020,14 +1071,14 @@ class ReaxFF_nn(object):
          self.Loss  += self.ME*self.lambda_me
       self.accuracy  = self.accuracy/self.nmol
 
-  def set_neurons(self):
+  def set_variable_list(self):
       self.unit = 4.3364432032e-2
       self.p_g  = ['boc1','boc2','coa2','ovun6','lp1',#'lp3',
                    'ovun7','ovun8','val6','tor2',
                    'tor3','tor4','cot2','coa4','ovun4',               # 
                    'ovun3','val8','val9','val10',
                    'coa3','pen2','pen3','pen4','vdw1',
-                   'cutoff','hbtol','acut'] # # 
+                   'cutoff','acut'] # # 'hbtol',
                    # 'trip2','trip1','trip4','trip3' ,'swa','swb'
                    # tor3,tor4>0 
 
@@ -1086,7 +1137,7 @@ class ReaxFF_nn(object):
       if self.optword.find('novdw')>=0:
          cons = cons + ['gammaw','vdw1','rvdw','Devdw','alfa'] 
       if self.optword.find('nohb')>=0:
-         cons = cons + ['Dehb','rohb','hb1','hb2','hbtol'] 
+         cons = cons + ['Dehb','rohb','hb1','hb2'] #,'hbtol'
 
       self.tor_v = ['tor2','tor3','tor4','V1','V2','V3','tor1','cot1','cot2'] 
 
@@ -1165,8 +1216,8 @@ class ReaxFF_nn(object):
       self.get_rcbo()
       self.m = set_matrix(self.m_,self.spec,self.bonds,
                           self.mfopt,self.mpopt,self.bdopt,self.messages,
-                          self.bo_layer,self.bo_layer_,0,0,
-                          self.mf_layer,self.mf_layer_,3,3,
+                          (6,0),(6,0),0,0,
+                          self.mf_layer,self.mf_layer_,self.MessageFunction_,self.MessageFunction,
                           self.be_layer,self.be_layer_,1,1,
                           (9,0),(9,0),1,1,
                           None,self.be_universal_nn,self.mf_universal_nn,None)
@@ -1181,7 +1232,7 @@ class ReaxFF_nn(object):
       self.ea_var = {}        # parameter list to be optimized with evolutional algrithom
       for k in self.var:
             key = k.split('_')[0]
-            if key in self.eaopt:
+            if key in self.eaopt or k in self.eaopt:
                self.ea_var[k] = self.p_[k]
 
       if self.clip_op:
@@ -1200,7 +1251,7 @@ class ReaxFF_nn(object):
       self.m = set_matrix(self.m_,self.spec,self.bonds,
                           self.mfopt,self.mpopt,self.bdopt,self.messages,
                           (6,0),(6,0),0,0,
-                          self.mf_layer,self.mf_layer_,3,3,
+                          self.mf_layer,self.mf_layer_,self.MessageFunction_,self.MessageFunction,
                           self.be_layer,self.be_layer_,1,1,
                           (9,0),(9,0),1,1,
                           None,self.be_universal_nn,self.mf_universal_nn,None)
@@ -1414,16 +1465,11 @@ class ReaxFF_nn(object):
                 if kkk[2]=='H':
                    hasH=True
           if p is not None:
-             if (k in self.opt or key in self.opt) and (key not in self.cons and k not in self.cons):
-             #   p_ = self.p_[key]*self.unit if k in self.punit else self.p_[key]
-             #   upop.append(tf.compat.v1.assign(self.var[key],p_))
-             # else:
-                if key in p:
-                   p_ = p[key]*self.unit if k in self.punit else p[key]
+             if key in p:
+                p_ = p[key]*self.unit if k in self.punit else p[key]
+                if (k in self.opt or key in self.opt) and (key not in self.cons and k not in self.cons):
                    if not hasH: upop.append(tf.compat.v1.assign(self.var[key],p_))
-             elif key in self.ea_var:
-                if key in p:
-                   p_ = p[key]*self.unit if k in self.punit else p[key]
+                elif key in self.ea_var:
                    self.feed_dict[self.var[key]] = p_
                    self.ea_var[key]              = p[key]
       
@@ -1461,8 +1507,10 @@ class ReaxFF_nn(object):
          self.session(learning_rate=learning_rate,method=method)  
 
       libfile = self.libfile.split('.')[0]
+      totrain = True
+      i       = 0
 
-      for i in range(step+1):
+      while totrain:
           if i==0:
              loss,lpenalty,self.ME_,accu,accs   = self.sess.run([self.Loss,
                                                       self.loss_penalty,
@@ -1520,6 +1568,10 @@ class ReaxFF_nn(object):
                 # E,dfte,zpe = self.sess.run([self.E,self.dft_energy,self.zpe],
                 #                           feed_dict=self.feed_dict)
                 # self.plot_result(i,E,dfte)
+                if accu>self.convergence:
+                   totrain = False
+                else:
+                   i = 0
              if not close_session:
                 if i<=200:
                    _loss = loss_
@@ -1538,7 +1590,7 @@ class ReaxFF_nn(object):
              self.write_lib(libfile=libfile,loss=loss_)
              print('-  Convergence Occurred, job compeleted.')
              break
-      
+          i += 1
       self.get_pentalty()
       self.loss_ = loss_ if not (np.isnan(loss) or np.isinf(loss)) else 9999999.9
       if self.loss_ < 9999999.0: self.write_lib(libfile=libfile,loss=loss_)
@@ -1664,7 +1716,7 @@ class ReaxFF_nn(object):
               'score':score,
               'BOFunction':0,#self.BOFunction,
               'EnergyFunction':1,# self.EnergyFunction,
-              'MessageFunction':3,# self.MessageFunction, 
+              'MessageFunction': self.MessageFunction, 
               'VdwFunction':1,#self.VdwFunction,
               'messages':self.messages,
               'bo_layer':self.bo_layer,
@@ -1695,7 +1747,7 @@ class ReaxFF_nn(object):
          self.m_  = j['m']
          # self.BOFunction_      = j['BOFunction']
          # self.EnergyFunction_  = j['EnergyFunction'] 
-         # self.MessageFunction_ = j['MessageFunction']
+         self.MessageFunction_ = j['MessageFunction']
          # self.VdwFunction_     = j['VdwFunction']
          self.MolEnergy_         = j['MolEnergy']
          # self.bo_layer_        = j['bo_layer']
@@ -1717,8 +1769,6 @@ class ReaxFF_nn(object):
       ''' adding some penalty term to accelerate the training '''
       log_    = -9.21044036697651
       penalty = 0.0
-      pen_w   = 0.0
-      pen_b   = 0.0
       wb_p    = []
       if self.regularize_be:
          wb_p.append('fe')
@@ -1736,11 +1786,14 @@ class ReaxFF_nn(object):
              wb_message.append('fm')          
              layer['fm'] = self.mf_layer[1]  
 
-      self.penalty_bop = {}
+      self.penalty_bop     = {}
+      self.penalty_bo      = {}
       self.penalty_bo_rcut = {}
-      self.penalty_be_cut = {}
-      self.penalty_rcut = {}
-      self.penalty_ang  = {}
+      self.penalty_be_cut  = {}
+      self.penalty_rcut    = {}
+      self.penalty_ang     = {}
+      self.penalty_w       = tf.constant(0.0)
+      self.penalty_b       = tf.constant(0.0)
 
       for bd in self.bonds: 
           atomi,atomj = bd.split('-') 
@@ -1763,6 +1816,8 @@ class ReaxFF_nn(object):
           self.penalty_bop[bd]     = tf.constant(0.0)
           self.penalty_be_cut[bd]  = tf.constant(0.0)
           self.penalty_bo_rcut[bd] = tf.constant(0.0)
+          self.penalty_bo[bd]      = tf.constant(0.0)
+
           for mol in self.mols:
               if self.nbd[mol][bd]>0:       
                  b_   = self.b[mol][bd]
@@ -1776,8 +1831,25 @@ class ReaxFF_nn(object):
                  fao  = tf.where(tf.greater(self.rbd_[mol][bd],self.rcuta[bd]),1.0,0.0)  ##### r> rcuta that bo = 0.0
                  self.penalty_bo_rcut[bd] += tf.reduce_sum(bo0_*fao)
 
-                 fesi = tf.where(tf.less_equal(bop_,self.botol),1.0,0.0)                 ##### bo <= 0.0 that e = 0.0
+                 fesi = tf.where(tf.less_equal(bo0_,self.botol),1.0,0.0)                 ##### bo <= 0.0 that e = 0.0
                  self.penalty_be_cut[bd]  += tf.reduce_sum(tf.nn.relu(self.esi[mol][bd]*fesi))
+                 
+                 if self.bo_clip:
+                     if (bd in self.bo_clip) or (bdr in self.bo_clip):
+                        bd_  = bd if bd in self.bo_clip else bdr
+                     for sbo in self.bo_clip[bd_]:
+                         r,d_i,d_j,bo_l,bo_u = sbo
+                         fe   = tf.where(tf.logical_and(tf.less_equal(self.rbd[bd],r),
+                                                         tf.logical_and(tf.greater_equal(self.Dbi[bd],d_i),
+                                                                        tf.greater_equal(self.Dbj[bd],d_j))),
+                                          1.0,0.0)   ##### r< r_e that bo > bore_
+                         self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((bo_l-self.esi[bd])*fe))
+                                                                                          # self.bo0[bd]
+                         fe   = tf.where(tf.logical_and(tf.greater_equal(self.rbd[bd],r),
+                                                         tf.logical_and(tf.greater_equal(self.Dbi[bd],d_i),
+                                                                        tf.greater_equal(self.Dbj[bd],d_j))),
+                                          1.0,0.0)  ##### r> r_e that bo < bore_
+                         self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd]-bo_u)*fe))
 
               if self.spv_ang:
                  self.penalty_ang[mol] = tf.reduce_sum(self.thet2[mol]*self.fijk[mol])
@@ -1785,46 +1857,49 @@ class ReaxFF_nn(object):
           penalty  = tf.add(self.penalty_be_cut[bd]*self.lambda_bd,penalty)
           penalty  = tf.add(self.penalty_bop[bd]*self.lambda_bd,penalty)        
           penalty  = tf.add(self.penalty_bo_rcut[bd]*self.lambda_bd,penalty)
+          penalty  = tf.add(self.penalty_bo[bd]*self.lambda_bd,penalty)   
 
           # penalize term for regularization of the neural networs
           if self.regularize:                             # regularize to avoid overfit
              for k in wb_p:
                  for k_ in w_n:
                      key     = k + k_ + '_' + bd
-                     pen_w  += tf.reduce_sum(tf.square(self.m[key]))
+                     self.penalty_w  += tf.reduce_sum(tf.square(self.m[key]))
                  if self.regularize_bias:
                     for k_ in b_n:
                         key     = k + k_ + '_' + bd
-                        pen_b  += tf.reduce_sum(tf.square(self.m[key]))
+                        self.penalty_b  += tf.reduce_sum(tf.square(self.m[key]))
                  for l in range(layer[k]):                                               
-                     pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+bd][l]))
+                     self.penalty_w += tf.reduce_sum(tf.square(self.m[k+'w_'+bd][l]))
                      if self.regularize_bias:
-                        pen_b += tf.reduce_sum(tf.square(self.m[k+'b_'+bd][l]))
+                        self.penalty_b += tf.reduce_sum(tf.square(self.m[k+'b_'+bd][l]))
 
       if self.regularize:                              # regularize
          for sp in self.spec:
              for k in wb_message:
                  for k_ in w_n:
                      key     = k + k_ + '_' + sp
-                     pen_w  += tf.reduce_sum(tf.square(self.m[key]))
+                     self.penalty_w  += tf.reduce_sum(tf.square(self.m[key]))
                  if self.regularize_bias:
                     for k_ in b_n:
                         key     = k + k_ + '_' + sp
-                        pen_b  += tf.reduce_sum(tf.square(self.m[key]))
+                        self.penalty_b  += tf.reduce_sum(tf.square(self.m[key]))
                  for l in range(layer[k]):                                               
-                     pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+sp][l]))
+                     self.penalty_w += tf.reduce_sum(tf.square(self.m[k+'w_'+sp][l]))
                      if self.regularize_bias:
-                        pen_b += tf.reduce_sum(tf.square(self.m[k+'b_'+sp][l]))
-         penalty = tf.add(self.lambda_reg*pen_w,penalty)
-         penalty = tf.add(self.lambda_reg*pen_b,penalty)
+                        self.penalty_b += tf.reduce_sum(tf.square(self.m[k+'b_'+sp][l]))
+         penalty = tf.add(self.lambda_reg*self.penalty_w,penalty)
+         penalty = tf.add(self.lambda_reg*self.penalty_b,penalty)
       return penalty
 
   def get_pentalty(self):
       (penalty_bop,penalty_bo_rcut,
-          penalty_be_cut,
-          penalty_rcut,rc_bo) = self.sess.run([self.penalty_bop,self.penalty_bo_rcut,
-                                         self.penalty_be_cut,
-                                         self.penalty_rcut,self.rc_bo],
+          penalty_bo,penalty_be_cut,
+          penalty_rcut,rc_bo,
+          penalty_w,penalty_b) = self.sess.run([self.penalty_bop,self.penalty_bo_rcut,
+                                         self.penalty_bo,self.penalty_be_cut,
+                                         self.penalty_rcut,self.rc_bo,
+                                         self.penalty_w,self.penalty_b],
                                          feed_dict=self.feed_dict)
       rcut = self.rcut
       print('\n------------------------------------------------------------------------')
@@ -1835,8 +1910,8 @@ class ReaxFF_nn(object):
       for bd in self.bonds:
           if bd in penalty_bop:
              print('bop cutoff penalty of                             {:5s}: {:6.4f}'.format(bd,penalty_bop[bd]))
-         #  if bd in penalty_bo:
-         #     print('BO state penalty of                             {:5s}: {:6.4f}'.format(bd,penalty_bo[bd]))
+          if bd in penalty_bo:
+             print('BO state penalty of                               {:5s}: {:6.4f}'.format(bd,penalty_bo[bd]))
           if bd in penalty_bo_rcut:
              print('Differency between rcut-bo and rcut Penalty of    {:5s}: {:6.4f} {:6.4f} {:6.4f}'.format(bd,penalty_bo_rcut[bd],rc_bo[bd],rcut[bd]))
           # if bd in penalty_esi:
@@ -1845,6 +1920,8 @@ class ReaxFF_nn(object):
              print('Bond-Energy at radius cutoff penalty of           {:5s}: {:6.4f}'.format(bd,penalty_be_cut[bd]))
           if bd in penalty_rcut:
              print('Bond-Order at radius cutoff penalty of            {:5s}: {:6.4f}'.format(bd,penalty_rcut[bd]))
+      print('Sum of square of weight:                          {:6.4f}'.format(penalty_w))
+      print('Sum of square of bias:                            {:6.4f}'.format(penalty_b))
       print('\n')
      
       # print('\n------------------------------------------------------------------------')
