@@ -1,4 +1,3 @@
-from __future__ import print_function
 from os import system, getcwd, chdir,listdir
 from os.path import isfile,exists,isdir
 import numpy as np
@@ -12,12 +11,12 @@ p_name = ['boc1','boc2','coa2','trip4','trip3','kc2','ovun6','trip2',
          'val9','val10','n.u.','pen2','pen3','pen4','n.u.','tor2',
          'tor3','tor4','n.u.','cot2','vdw1','cutoff','coa4','ovun4',
          'ovun3','val8','acut','hbtol','n.u.','n.u.','coa3']
-         
+
 line_spec = []
 line_spec.append(['rosi','val','mass','rvdw','Devdw','gamma','ropi','vale'])
-line_spec.append(['alfa','gammaw','valang','ovun5','n.u.','chi','mu','atomic'])
+line_spec.append(['alfa','gammaw','valang','ovun5','n.u.','chi','mu','hbond'])
 line_spec.append(['ropp','lp2','n.u.','boc4','boc3','boc5','n.u.','n.u.'])
-line_spec.append(['ovun2','val3','n.u.','valboc','val5','n.u.','n.u.','n.u.'])
+line_spec.append(['ovun2','val3','n.u.','valboc','val5','n.u.','n.u.','atomic'])
 
 line_bond = []
 line_bond.append(['Desi','Depi','Depp','be1','bo5','corr13','bo6','ovun1'])
@@ -50,13 +49,13 @@ def read_ffield(p={},zpe=False,libfile='ffield',
           for i in range(int(len(lin)/2-1)):
               k = lin[i*2+1]
               try:
-                 zpe_[k] = float(lin[i*2])
+                 zpe_[k] = np.float32(lin[i*2])
               except ValueError:
                  zpe_[k] = 0.0
 
        for i in range(npar):
            pn = p_name[i]
-           p[pn] = float(lines[2+i].split()[0]) 
+           p[pn] = np.float32(lines[2+i].split()[0]) 
 
        # ---------   parameters for species   ---------
        nofc   = 1                 #  number of command line
@@ -70,7 +69,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
            for il,line in enumerate(line_spec):
                ls = 1 if il == 0 else 0
                for ip,pn in enumerate(line):
-                   p[pn+'_'+spec[i]] = np.float(lines[nofc+npar+nsc+i*nsl+il].split()[ls+ip])
+                   p[pn+'_'+spec[i]] = np.float32(lines[nofc+npar+nsc+i*nsl+il].split()[ls+ip])
 
        # ---------  parameters for bonds   ---------
        bonds = []
@@ -85,7 +84,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
            for il,line in enumerate(line_bond):
                ls = 2 if il == 0 else 0
                for ip,pn in enumerate(line):
-                   p[pn+'_'+bond] = np.float(lines[nofc+npar+nsc+nspec*nsl+nbc+i*nbl+il].split()[ls+ip])
+                   p[pn+'_'+bond] = np.float32(lines[nofc+npar+nsc+nspec*nsl+nbc+i*nbl+il].split()[ls+ip])
 
        # ---------   parameters for off-diagonal bonds   ---------
        offd  = []
@@ -99,7 +98,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
            bond = spec[b1-1] + '-' +spec[b2-1]
            offd.append(bond)
            for ip,pn in enumerate(line_offd):
-               p[pn+'_'+bond] = np.float(lines[nofc+npar+nsc+nspec*nsl+nbc+nbond*nbl+noc+i].split()[2+ip])
+               p[pn+'_'+bond] = np.float32(lines[nofc+npar+nsc+nspec*nsl+nbc+nbond*nbl+noc+i].split()[2+ip])
            
        # ---------   parameters for angles   ---------
        angs = []
@@ -115,7 +114,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
            if (not ang in angs) and (not angr in angs):
               angs.append(ang)
               for ip,pn in enumerate(line_ang):
-                  p[pn+'_'+ang] = np.float(l[3+ip])
+                  p[pn+'_'+ang] = np.float32(l[3+ip])
 
        # ---------   parameters for torsions   --------- 
        ntl  = 1 
@@ -137,7 +136,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
               (not tor1 in tors) and (not torr1 in tors):
               tors.append(tor)
               for ip,pn in enumerate(line_tor):
-                  p[pn+'_'+tor] = np.float(l[4+ip])
+                  p[pn+'_'+tor] = np.float32(l[4+ip])
 
        # ---------   parameters for HBs   ---------
        hbs = []
@@ -150,7 +149,7 @@ def read_ffield(p={},zpe=False,libfile='ffield',
            hb = spec[hb1-1] + '-' +spec[hb2-1] + '-' +spec[hb3-1]
            hbs.append(hb)
            for ip,pn in enumerate(line_hb):
-               p[pn+'_'+hb] = np.float(l[3+ip])
+               p[pn+'_'+hb] = np.float32(l[3+ip])
     else:
        print('-  Error: lib file is not found!')
        p = None
@@ -161,7 +160,7 @@ def write_ffield(p,spec,bonds,offd,angs,tors,hbs,zpe=None,libfile='ffield',
                  p_name=p_name,line_spec=line_spec,
                  line_bond=line_bond,line_offd=line_offd,
                  line_ang=line_ang,line_tor=line_tor,line_hb=line_hb,
-                 loss=None,
+                 m=None,mf_layer=(9,1),be_layer=(9,1),loss=None,rcut=None,
                  logo='!-ReaxFF-Parameter-From-Machine-Learning-Computational-Materials-Science-172-(2020)-109393'):
     flib = open(libfile,'w')
 
@@ -188,46 +187,50 @@ def write_ffield(p,spec,bonds,offd,angs,tors,hbs,zpe=None,libfile='ffield',
         else:
            print('%10.4f      ! %s ' %(p[pn],pn), file=flib)
     for i in range(len(line_spec)):
-        txt = '%d ! Nr of atoms; atomID;' %len(spec) if i==0 else '            '
+        txt = '{:d} ! Nr of atoms; atomID;'.format(len(spec)) if i==0 else '            '
         for ls in line_spec[i]:
             txt += ls + '; '
         print(txt,file=flib)
 
     for sp in spec:  # atomic species 
         for i,line in enumerate(line_spec):
-            txt = '%2s' %sp if i==0 else '  '
+            txt = '{:2s}'.format(sp) if i==0 else '  '
             for pn in line:
                 pn_ = pn+'_'+sp
                 if pn=='n.u.':
                    v = 0.0
                 else:
                    v = p[pn_]
-                txt += '%9.4f' %v
+                txt += '{:9.4f}'.format(v)
             print(txt,file=flib)
 
     # ---------   parameters for bonds   --------- 
     for i,line in enumerate(line_bond): 
-        txt = '%3d  ! Nr of bonds;' %len(bonds) if i==0  else '                   '
+        txt = '{:3d}  ! Nr of bonds;'.format(len(bonds)) if i==0  else '                   '
         for ls in line:
             txt += ls +'; '
         print(txt,file=flib)
     for bs in bonds:
         for i,line in enumerate(line_bond):
-            txt = '%3d%3d' %(spec.index(bs.split('-')[0])+1,spec.index(bs.split('-')[1])+1) if i==0  else '      '
+            txt = '{:3d}{:3d}'.format(spec.index(bs.split('-')[0])+1,spec.index(bs.split('-')[1])+1) if i==0  else '      '
             for pn in line:
                 pn_ = pn+'_'+bs
                 if pn=='n.u.':
                    v = 0.0
+                elif pn=='ovcorr':
+                   v = p[pn_] if m is None else 0.0
+                elif pn=='corr13':
+                   v = p[pn_] if m is None else 2.0
                 else:
                    if pn_ in p:
                       v = p[pn_]
                    else:
                       v = 0.0 # np.random.normal(loc=0.0, scale=0.2, size=0)
-                txt += '%9.4f' %v
+                txt += '{:9.4f}'.format(v)
             print(txt,file=flib)
 
     # ---------    parameters for off-diagonal bonds   --------- 
-    txt = '%3d  ! off-diagonal terms;' %len(offd) 
+    txt = '{:3d}  ! off-diagonal terms;'.format(len(offd)) 
     for ls in line_offd:
         txt += ls +'; '
     print(txt,file=flib)
@@ -238,36 +241,36 @@ def write_ffield(p,spec,bonds,offd,angs,tors,hbs,zpe=None,libfile='ffield',
            continue
         if b[0]==b[1]:
            continue
-        txt = '%3d%3d' %(spec.index(bs.split('-')[0])+1,spec.index(bs.split('-')[1])+1) 
+        txt = '{:3d}{:3d}'.format(spec.index(bs.split('-')[0])+1,spec.index(bs.split('-')[1])+1) 
         for pn in line_offd:
             pn_ = pn+'_'+bs
             if pn=='n.u.':
                v = 0.0
             else:
                v = p[pn_]
-            txt += '%9.4f' %v
+            txt += '{:9.4f}'.format(v)
         print(txt,file=flib)
 
     # ---------   parameters for angles   ---------
-    txt = '%3d  ! Nr of angles;' %len(angs) 
+    txt = '{:3d}  ! Nr of angles;'.format(len(angs)) 
     for ls in line_ang:
         txt += ls +'; '
     print(txt,file=flib)
 
     for a in angs:
         a_ = a.split('-')
-        txt = '%3d%3d%3d' %(spec.index(a_[0])+1,spec.index(a_[1])+1,spec.index(a_[2])+1)
+        txt = '{:3d}{:3d}{:3d}'.format(spec.index(a_[0])+1,spec.index(a_[1])+1,spec.index(a_[2])+1)
         for pn in line_ang:
             pn_ = pn+'_'+a
             if pn=='n.u.':
                v = 0.0
             else:
                v = p[pn_]
-            txt += '%9.4f' %v
+            txt += '{:9.4f}'.format(v)
         print(txt,file=flib)
 
     # ---------  parameters for tors   ---------
-    txt = '%3d  ! Nr of torsions;' %len(tors) 
+    txt = '{:3d}  ! Nr of torsions;'.format(len(tors)) 
     for ls in line_tor:
         txt += ls +'; '
     print(txt,file=flib)
@@ -278,32 +281,189 @@ def write_ffield(p,spec,bonds,offd,angs,tors,hbs,zpe=None,libfile='ffield',
         e2 = spec.index(aa[1])+1
         e3 = spec.index(aa[2])+1
         e4 = spec.index(aa[3])+1 if aa[3] != 'X' else 0
-        txt = '%3d%3d%3d%3d' %(e1,e2,e3,e4)
+        txt = '{:3d}{:3d}{:3d}{:3d}'.format(e1,e2,e3,e4)
         for pn in line_tor:
             pn_ = pn+'_'+a
             if pn=='n.u.':
                v = 0.0
             else:
                v = p[pn_]
-            txt += '%9.4f' %v
+            txt += '{:9.4f}'.format(v)
         print(txt,file=flib)
 
     # ---------  parameters for HBs   ---------
-    txt = '%3d  ! Nr of Hbonds;' %len(hbs) 
+    txt = '{:3d}  ! Nr of Hbonds;'.format(len(hbs)) 
     for ls in line_hb:
         txt += ls +'; '
     print(txt,file=flib)
 
     for a in hbs:
-        txt = '%3d%3d%3d' %(spec.index(a.split('-')[0])+1,spec.index(a.split('-')[1])+1,spec.index(a.split('-')[2])+1)
+        txt = '{:3d}{:3d}{:3d}'.format(spec.index(a.split('-')[0])+1,spec.index(a.split('-')[1])+1,spec.index(a.split('-')[2])+1)
         for pn in line_hb:
             pn_ = pn+'_'+a
             if pn=='n.u.':
                v = 0.0
             else:
                v = p[pn_]
-            txt += '%9.4f' %v
+            txt += '{:9.4f}'.format(v)
         print(txt,file=flib)
+
+    if m is not None:                        ### for neural net work parameters
+       shap = len(m['fmwo_'+spec[0]][0])
+       nin  = 3
+       nout = shap # shape of output layer
+       print('{:d}   !fnn_wi {:d} {:d}'.format(len(spec),mf_layer[0],nin),file=flib) ### weight
+       for sp in spec:
+           print(' {:2s} '.format(sp),end=' ',file=flib) 
+           for i in range(nin):
+               if i!=0:
+                  print('    ',end=' ',file=flib)
+               for j in range(mf_layer[0]):
+                   print('{:20.16f}'.format(m['fmwi_'+sp][i][j]),end=' ',file=flib)
+               print(' ',file=flib)
+       print('{:d}  !fnn_bi {:d}'.format(len(spec),mf_layer[0]),file=flib) ### bias 
+       for sp in spec:
+           print(' {:2s} '.format(sp),end=' ',file=flib) 
+           for j in range(mf_layer[0]):
+               print('{:20.16f}'.format(m['fmbi_'+sp][j]),end=' ',file=flib)
+           print(' ',file=flib)
+
+       if mf_layer[1]>0:
+          print('{:d}  !fnn_wh {:d} {:d}'.format(len(spec),mf_layer[0],mf_layer[0]),file=flib)
+          for sp in spec:
+              print(' {:2s} '.format(sp),end=' ',file=flib) 
+              for l in range(mf_layer[1]):
+                  for i in range(mf_layer[0]):
+                      if i!=0 :
+                         print('    ',end=' ',file=flib)
+                      for j in range(mf_layer[0]):
+                          print('{:20.16f}'.format(m['fmw_'+sp][l][i][j]),end=' ',file=flib)
+                      print(' ',file=flib)
+        
+          print('{:d}  !fnn_bh {:d}'.format(len(spec),mf_layer[0]),file=flib)
+          for sp in spec:
+              print(' {:2s} '.format(sp),end=' ',file=flib) 
+              for l in range(mf_layer[1]):
+                  if l!=0:
+                     print('     ',end=' ',file=flib)
+                  for j in range(mf_layer[0]):
+                      print('{:20.16f}'.format(m['fmb_'+sp][l][j]),end=' ',file=flib)
+                  print(' ',file=flib)
+
+       print('{:d}  !fnn_wo {:d} {:d}'.format(len(spec),nout,mf_layer[0]),file=flib)
+       for sp in spec:
+           print(' {:2s} '.format(sp),end=' ',file=flib) 
+           for i in range(mf_layer[0]):
+               if i!=0 and nout>1:
+                  print('    ',end=' ',file=flib)
+               for j in range(nout):
+                   print('{:20.16f}'.format(m['fmwo_'+sp][i][j]),end=' ',file=flib)
+               if nout>1:
+                  print(' ',file=flib)
+           if nout==1:
+              print(' ',file=flib) 
+
+       print('{:d}  !fnn_bo {:d}'.format(len(spec),nout),file=flib)
+       for sp in spec:
+           print(' {:2s} '.format(sp),end=' ',file=flib) 
+           for j in range(nout):
+               print('{:20.16f}'.format(m['fmbo_'+sp][j]),end=' ',file=flib)
+           print(' ',file=flib)
+
+       #print(' belayer {:d} {:d}'.format(be_layer[0],be_layer[1]),file=flib)
+       nin  = 3
+       nout = 1
+       print('{:d}  !enn_wi {:d} {:d}'.format(len(bonds),be_layer[0],nin),file=flib)
+       for bd in bonds:
+           b = bd.split('-') 
+           id1 = spec.index(b[0]) + 1
+           id2 = spec.index(b[1]) + 1
+           print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+           for i in range(nin):
+               if i!=0:
+                  print('       ',end=' ',file=flib)
+               for j in range(be_layer[0]):
+                   print('{:20.16f}'.format(m['fewi_'+bd][i][j]),end=' ',file=flib)
+               print(' ',file=flib)
+               
+       print('{:d}  !enn_bi {:d}'.format(len(bonds),be_layer[0]),file=flib)
+       for bd in bonds:
+           b = bd.split('-') 
+           id1 = spec.index(b[0]) + 1
+           id2 = spec.index(b[1]) + 1
+           print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+           for j in range(be_layer[0]):
+               print('{:20.16f}'.format(m['febi_'+bd][j]),end=' ',file=flib)
+           print(' ',file=flib)
+
+       if be_layer[1]>0:
+          print('{:d}  !enn_wh {:d} {:d}'.format(len(bonds),be_layer[0],be_layer[0]),file=flib)
+          for bd in bonds:
+              b = bd.split('-') 
+              id1 = spec.index(b[0]) + 1
+              id2 = spec.index(b[1]) + 1
+              print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+              for l in range(be_layer[1]):
+                  for i in range(be_layer[0]):
+                      if i!=0:
+                         print('       ',end=' ',file=flib)
+                      for j in range(be_layer[0]):
+                          print('{:20.16f}'.format(m['few_'+bd][l][i][j]),end=' ',file=flib)
+                      print(' ',file=flib)
+
+          print('{:d}  !enn_bh {:d}'.format(len(bonds),be_layer[0]),file=flib)
+          for bd in bonds:
+              b = bd.split('-') 
+              id1 = spec.index(b[0]) + 1
+              id2 = spec.index(b[1]) + 1
+              print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+              for l in range(be_layer[1]):
+                  if l!=0:
+                     print('         ',end=' ',file=flib)
+                  for j in range(be_layer[0]):
+                      print('{:20.16f}'.format(m['feb_'+bd][l][j]),end=' ',file=flib)
+                  print(' ',file=flib)
+
+       print('{:d}  !enn_wo {:d} {:d}'.format(len(bonds),nout,be_layer[0]),file=flib)
+       for bd in bonds:
+           b = bd.split('-') 
+           id1 = spec.index(b[0]) + 1
+           id2 = spec.index(b[1]) + 1
+           print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+           for i in range(be_layer[0]):
+               if i!=0 and nout>1:
+                  print('           ',end=' ',file=flib)
+               for j in range(nout):
+                   print('{:20.16f}'.format(m['fewo_'+bd][i][j]),end=' ',file=flib)
+               if nout>1:
+                  print(' ',file=flib)
+           if nout==1:
+              print(' ',file=flib) 
+
+       print('{:d}  !enn_bo {:d}'.format(len(bonds),nout),file=flib)
+       for bd in bonds:
+           b = bd.split('-') 
+           id1 = spec.index(b[0]) + 1
+           id2 = spec.index(b[1]) + 1
+           print(' {:2d} {:2d} '.format(id1,id2),end=' ',file=flib) 
+           for j in range(nout):
+               print('{:20.16f}'.format(m['febo_'+bd][j]),end=' ',file=flib)
+           print(' ',file=flib)
+
+       with open('control','w') as fc:
+           print('tabulate_long_range	0 ! denotes the granularity of long range tabulation, 0 means no tabulation',file=fc)
+           if rcut is None:
+              mrcut = 3.5
+           else:
+              mrcut = max(rcut.values())
+           print('nbrhood_cutoff	    {:f}  ! near neighbors cutoff for bond calculations'.format(mrcut),file=fc)
+           print('bond_graph_cutoff	    0.3 ! bond strength cutoff for bond graphs',file=fc)
+           print('hbond_cutoff	        7.5 ! cutoff distance for hydrogen bond interactions',file=fc)
+           print('thb_cutoff		    {:f} ! cutoff value for three body interactions'.format(p['acut']),file=fc)
+           print('mflayer_m		        {:d} ! neural network width'.format(mf_layer[0]),file=fc)
+           print('mflayer_n		        {:d} ! number of hidden layer'.format(mf_layer[1]),file=fc)
+           print('belayer_m		        {:d} ! neural network width'.format(be_layer[0]),file=fc)
+           print('belayer_n		        {:d} ! number of hidden layer'.format(be_layer[1]),file=fc)
     flib.close()
 
 def write_lib(p,spec,bonds,offd,angs,tors,hbs,
