@@ -12,7 +12,7 @@ def structure(gen):
     elif isfile(_root+'/'+gen+'.cif'):
        A = read(_root+'/'+gen+'.cif')
     else:
-       raise IOError(' * Structure is not found.')
+       raise RuntimeError('Structure is not found!')
     return A
 
 
